-- ************************************ Task System ************************************ ''--
TaskSystem = getClass("TaskSystem")

function TaskSystem:addTasks(t)
	t = self:wrapSingleTable(t)
	local r = {}
	for k,v in pairs(t) do
		if (isTable(v)) then
			local task = self:addTask(v)
			if (task) then
				table.insert(r, task)
			end
		end
	end
	return r
end

function TaskSystem:wrapSingleTable(t)
	GH.assert(isTable(t))
	for k,v in pairs(t) do
		if (type(k) ~= "number" or type(v) ~= "table") then
			return { t }
		end
	end
	return t
end

function TaskSystem:onTaskStart(task, ...)
	if (task.actor and task.affectsGameplay ~= false and task.shouldCheckForCompound ~= false) then
		local tray = level and level.getTray and level:getTray(task.actor)
		if (tray) then
			tray:checkCompoundIngredients(task.actor)
		end
	end
end

function TaskSystem:onTaskFinish(task, ...)
end

function TaskSystem:onTaskCancel(task, ...)
end

function TaskSystem:getAnimationDesc(animId)
	GH.assert(isString(animId), "Must provide an animation id")
	if (animationsList) then
		for _,v in pairs(animationsList) do
			if (isTable(v) and v.id == animId) then
				return v
			end
		end
	end
end

function TaskSystem:createSequentialAnimation(taskList, name, actors, exclusive, allowAutoSkip)
	GH.assert(actors == nil or isTable(actors))
	local treeDesc = {}
	for k,_ in ipairs(taskList) do
		table.push_back(treeDesc, k)
	end

	root = self:newAnimation({name=name, exclusive=exclusive, allowAutoSkip=allowAutoSkip}, actors)
	self:createTaskTree(name, root, treeDesc, taskList, actors, false)
end

-- Create a (sub) tree from an animation description
function TaskSystem:createTaskTree(animId, root, treeDesc, list, values, simultaneous)
	local curTask = root
	-- Loop over all the tasks in the animation
	for k,treeItem in pairs(treeDesc) do

		-- Determine whether we should be adding the next task
		-- alongside or after the previous task
		local isRoot = curTask == root

		-- If the tasks are located another level deep
		if (isTable(treeItem)) then

			-- Create a new group
			local group
			if (isRoot) then
				group = curTask:addGroup()
			elseif (simultaneous) then
				group = curTask:alsoGroup()
			else
				group = curTask:atEndGroup()
			end

			-- Call this function recursively
			self:createTaskTree(animId, group, treeItem, list, values, not simultaneous)

			-- Set the created group as the current task
			curTask = group

		-- Look up the task definition in the list of tasks
		else
			-- This needs to be an index into the animation description list
			GH.assert(isNumber(treeItem), "This is not a number: "..tostring(treeItem).." ("..tostring(k)..")")

			-- Make sure the provided number actually points to
			-- an animtion description in the list
			if (not isTable(list[treeItem])) then
				--print("Warning: "..treeItem.." is not a valid index in animation '"..animTable.name.."'")

			else
				-- Make a copy of the definition
				local desc = table.copy(list[treeItem])
				if (desc.taskEnabled ~= false) then

					-- Replace value keywords with actual values
					for dk,dv in pairs(desc) do
						local value = values[dv]
						if (value ~= nil and not isNumber(dv)) then
							desc[dk] = value
							--print(table.val_to_str(dk).." = "..table.val_to_str(dv).." => "..tostring(value))
						end
					end

					desc.isScripted = true
					desc.animationID = treeItem
					desc.animName = animId

					-- Actually create the task and add the task to the current tree
					local task
					if (isRoot) then
						task = curTask:add(desc)
					elseif (simultaneous) or (desc.instaPlay == true) then
						task = curTask:also(desc)
					else
						task = curTask:atEnd(desc)
					end

					if (debug) then
						local debugInformation = ""
						if (player) then
							debugInformation = debugInformation.."episode: "..tostring(player.episode).." shift: "..tostring(player.shift).." "
						end
						task.debugInformation = debugInformation.."animation: '"..animId.."' task: "..tostring(treeItem)
					end

					-- Set the created task as the current task
					curTask = task
				end
			end
		end

		first = false
	end
end

-- Create an animation stored in the animations table for this level
-- The named parameter table can contain value keyword substitutes
function TaskSystem:createAnimation(animId, paramTable, root)
	if (not isString(animId) or animId == "") then
		--print("Not an id: "..tostring(animId))
		return nil
	end

	local animTable = self:getAnimationDesc(animId)
	if (not isTable(animTable)) then
		return
	end

	if (not animTable.name or animTable.name == "") then
		animTable.name = animId
	end

	-- Resolve all values using predefined keywords and
	-- the parameter table
	local values = {}
	if (isTable(paramTable)) then
		for k,v in pairs(paramTable) do
			values[k] = v
		end
	end
	if (isTable(animTable.actors)) then
		for k,v in pairs(animTable.actors) do
			values[k] = level:resolveActor(v, paramTable)
			--print("Actor: "..table.val_to_str(k).." = "..table.val_to_str(values[k]))
			GH.assert(values[k] ~= nil, tostring(v).." is not a value ("..tostring(k)..")")
		end
	end

	-- Create a new animation in the task system
	if (root) then
		root:setup(animTable)
		-- TODO: what to do with 'values' (actors)
	else
		root = self:newAnimation(animTable, values)
	end

	root.musicMoodStart = animTable.musicMoodStart
	root.musicMoodEnd = animTable.musicMoodEnd

	-- Build the animation tree
	self:createTaskTree(animId, root, animTable.tree, animTable.list, values, false)

	return root
end

function TaskSystem:onAnimationStarted(anim)
--'print("["..ts(anim.id).."] Started: " .. ts(anim.name))
	if (level) then
		level:onAnimationStarted(anim)
	end
end

function TaskSystem:onAnimationFinished(anim)
--'print("["..ts(anim.id).."] Finished: " .. ts(anim.name))
	if (level) then
		level:onAnimationFinished(anim)
	end
end
