DropperObject = getClass("DropperObject")

function DropperObject:init(desc)
  DropperObject.superInit(self, desc)

  self.valueMultipliers = {}

  -- Patch for the "trough" upgrade to work:
  self.valueMultipliers.dropDelayMultiplier = 1.0

  self.dropMinDelay = desc.dropMinDelay or 8000
  self.dropMaxDelay = desc.dropMaxDelay or 12000

  self.objectSet = false
  self.firstTimeWalk = true
  self.levelFinishChecked = false
  self.objectLeftLevel = false
  self.heroIsWorking = false
  self.tasksCancelled = false

  self.enabled = desc.enabled or false

  self:GatherEditorParameters(desc)
  self:SetupObject(desc)
end

function DropperObject:onPreInitializeLevel()
  if self.enableStoryPosition == true then
    self:GoAndWaitInStoryPosition()
  end
end

function DropperObject:onLevelInitialized()
  DropperObject.super.onLevelInitialized(self)

  -- Patch for the "trough" upgrade to work:
  self.dropMinDelay = self.dropMinDelay * self.valueMultipliers.dropDelayMultiplier
  self.dropMaxDelay = self.dropMaxDelay * self.valueMultipliers.dropDelayMultiplier

  -- First time drop delay:
  self.dropDelay = math.Random(self.dropMinDelay, self.dropMaxDelay)
end

function DropperObject:ForceStar()
    self.enabled = true
    taskSystem:cancelAllTasksFor(self)
    self:Walk()
end

function DropperObject:ForceStop()
    self.enabled = false
    taskSystem:cancelAllTasksFor(self)
end

function DropperObject:onTick(time)
  self:tick(time)
  self.currentTime = time

  if (not self.enabled) then
      return
  end

  if level.isPlaying and self.firstTimeWalk == true then
    -- Cancel the current task (going to story location) and start the logic:
    self:ForceStar()
    self.firstTimeWalk = false
  end

  if self.levelFinishChecked == false then
    self.levelFinishChecked = self:CheckIfLevelFinished()
  elseif self.objectLeftLevel == false and self.moveObjectAfterGameplay == true then
    self:addTask({class = "WalkToTask",
      target = self.objectBackstagePosition,
      playDefaultOnFinish = false,
      ignoreBlocking = self.ignoreBlocking,
      })
    self:addTask({class = "CallTask", func = function() self.objectLeftLevel = true end})
    return
  elseif (self.tasksCancelled == false) then
    taskSystem:cancelAllTasksFor(self)
    self.tasksCancelled = true
  end

  if level.isPlaying and self.objectSet == true then
    self:PlayOccasionalSound() -- Play an occasional sound if configured

    self:UpdateDroppingLogic(time) -- Update the whole dropping process
    self:UpdateActionLogic(time) -- Update the selected action's process
    self:UpdateDropCleaning(time) -- Update decayed drops' state
  end
end

function DropperObject:CheckIfLevelFinished()
  if level.hasShiftFinished == true then
    taskSystem:cancelAllTasksFor(self)
  end
  return level.hasShiftFinished
end

function DropperObject:GoAndWaitInStoryPosition()
  if self.location ~= self.storyLocation then
    self:Walk(false, false, self.storyLocation)
  end
  -- Keep waiting for the level to start:
  self:Wait(function() self:GoAndWaitInStoryPosition() end)
end

function DropperObject:PlayOccasionalSound()
  -- Plays an occasional sound if enabled, every [x, x+n] milliseconds:
  if level.isPlaying and self.soundOccasional ~= "" then
    self.soundTime = self.soundTime - self.currentTime
    if self.soundTime <= 0 then
      self:playSample(self.soundOccasional)
      self.soundTime = math.Random(self.soundOccasionalMinDelay, self.soundOccasionalMaxDelay)
    end
  end
end

function DropperObject:UpdateDroppingLogic(time)
  if (self.dropEnabled == true and level.isPlaying == true) then
    -- If it's about time to drop something and it can be done...
    if (self.dropTime >= self.dropDelay
      and self.currentAction ~= "Walk"
      and self.needsToDrop == false
      and ((self.numDropsInScene < self.dropMaxNum)
        or (self.dropMaxNum == -1))) then
      self.needsToDrop = true
      -- Choose the perfect action to reach a free dropping spot (smart mode):
      if self.dropSmartMode == true then
        if self.currentTarget.isDroppable == true and
          #self.currentTarget.currentDrops < self.currentTarget.maxDropsInHere then
          self:PrepareNewDrop()
        else
          self:CancelAndWalkForDrop()
        end
      else
        -- ... or start the dropping action (smart mode disabled):
        self:PrepareNewDrop()
      end
    end

    -- Update the dropping time:
    if self.needsToDrop == false then
      self.dropTime = self.dropTime + time
    end
  end
end

function DropperObject:CancelAndWalkForDrop()
  taskSystem:cancelAllTasksFor(self)
  self.activityTime = 0
  self:Walk(true, true, nil)
end

function DropperObject:UpdateActionLogic(time)
  -- If current state is "Wait" we force the object to go for a walk:
  if self.currentAction == "Wait" then
    self.activityTime = self.activityTime + time
    if self.activityTime >= self.waitDelay then
      self.previousAction = "Wait"
      self:Walk(false, false, nil)
      self.activityTime = 0
    end
  -- Otherwise we check if we are waiting for a new drop (smart mode) or directly
  -- call ChooseNewAction in order to find another suitable action:
  elseif self.currentAction == "Walk" then
    self.activityTime = self.activityTime + time
    if (
      self.location and self.currentTarget and
      self.location.x == self.currentTarget.x and
      self.location.y == self.currentTarget.y) then
      if self.dropWhenArrive == true then
        --self:PrepareNewDrop()
        self.dropWhenArrive = false
      end
      self.previousAction = "Walk"
      self:ChooseNextAction()
      self.activityTime = 0
    end
  end
end

function DropperObject:UpdateDropCleaning(time)
  -- Remove the decayed drops from the level:
  self:DeleteObsoleteDrops()

  -- Add the new requested drops to the level, after being prepared:
  if self.isDropping == true then
    self.dropCurrentWaitingTime = self.dropCurrentWaitingTime + time
    if self.dropCurrentWaitingTime >= self.dropWaitingTime then
      if self.currentTarget and self.currentTarget.isDroppable == true and
        #self.currentTarget.currentDrops < self.currentTarget.maxDropsInHere then
          if self.heroIsWorking == false then
            self:CreateNewDrop()
            self.dropCurrentWaitingTime = 0
          end
      end
    end
  end
end

function DropperObject:GatherEditorParameters(desc)
  -- Gather and error control all the parameters coming from the editor:
  self:GatherLocations(desc)
  self:GatherDelaysAndChances(desc)
  self:GatherIdleAnimations(desc)
  self:GatherDropVariables(desc)
  self:GatherSoundVariables(desc)
  self:GatherOtherVariables(desc)
end

function DropperObject:GatherLocations(desc)
  self.enableStoryPosition = desc.enableStoryPosition or false
  self.storyLocation = desc.storyPosition or self.location
  self.moveObjectAfterGameplay = desc.moveObjectAfterGameplay or false
  self.objectBackstagePosition = desc.objectBackstagePosition or self.location

  self.locations = desc.locations or {}
  self.dropIndexes = desc.dropIndexes or {}
end

function DropperObject:GatherDelaysAndChances(desc)
  self.waitChance = desc.waitChance or 50
  self.walkChance = desc.walkChance or 50

  self.idleMinDelay = desc.idleMinDelay or 2000
  self.idleMaxDelay = desc.idleMaxDelay or 4000
  self.idleAnimChance = desc.idleAnimChance or 60

  self.waitMinDuration = desc.waitMinDuration or 5000
  self.waitMaxDuration = desc.waitMaxDuration or 10000
end

function DropperObject:GatherIdleAnimations(desc)
  self.idleAnimations = {}

  table.insert(self.idleAnimations,
    {animation = (desc.idleAnimation1 or self.baseAnimation),
     cycles = (desc.idleAnimation1Cycles or 1),
     frameTime = (desc.idleAnimation1FrameTime or 0)})

  table.insert(self.idleAnimations,
    {animation = (desc.idleAnimation2 or self.baseAnimation),
     cycles = (desc.idleAnimation2Cycles or 1),
     frameTime = (desc.idleAnimation2FrameTime or 0)})

  table.insert(self.idleAnimations,
    {animation = (desc.idleAnimation3 or self.baseAnimation),
     cycles = (desc.idleAnimation3Cycles or 1),
     frameTime = (desc.idleAnimation3FrameTime or 0)})
end

function DropperObject:GatherDropVariables(desc)
  self.dropEnabled = desc.dropEnabled ~= false -- Default true
  self.dropSmartMode = desc.dropSmartMode ~= false -- Default true
  self.dropMaxNum = desc.dropMaxNum or -1

  self.dropTimeout = desc.dropTimeout or 50000
  self.dropWaitingTime = desc.dropWaitingTime or 800

  self.dropAnimation = desc.dropAnimation or self.baseAnimation
  self.dropAnimationCycles = desc.dropAnimationCycles or 1
  self.dropAnimationFrameTime = desc.dropAnimationFrameTime or 0
  self.dropAnimationVisibleFrame = desc.dropAnimationVisibleFrame or 0

  self.dropAnchor = desc.dropAnchor or "bottom"
  self.dropOffset = desc.dropOffset or {x = 0.0, y = 0.0}
  self.dropFloorBehind = desc.dropFloorBehind ~= false -- Default true
  self.dropFloor = desc.dropFloorManual or 0

  self.dropWorkDirection = desc.dropWorkDirection or "left"
  self.dropWorkDuration = desc.dropWorkDuration or 0
  self.dropWorkOffset = desc.dropWorkOffset or {x = 0, y = 0}
  self.dropMinRotation = desc.dropMinRotation or -45
  self.dropMaxRotation = desc.dropMaxRotation or 45
  self.dropWorkAnimation = desc.dropWorkAnimation or "work"
  self.dropWorkFrameTime = desc.dropWorkFrameTime or -1
  self.dropWorkCharacter = desc.dropWorkCharacter or hero

  self.dropStockWorth = desc.dropStockWorth or 1
  self.dropProductGroup = desc.dropProductGroup or ""
  self.dropProductPart = desc.dropProductPart or 1

  self.dropShadow = desc.dropShadow or ""
  self.dropShadowBlendMode = desc.dropShadowBlendMode or "multiply"
  self.dropNotify = desc.dropNotify or ""
  self.dropProductSection = desc.dropProductSection or ""
  self.dropProductName = desc.dropProductName or ""
  hitMaskThreshold = desc.dropHitMaskThreshold or -1.0

  self.dropUseWorkChararcterIfUgrade = desc.dropUseWorkChararcterIfUgrade or false
  self.dropBoughtUpgrade = desc.dropBoughtUpgrade or ""
  self.workCharacterUpgrade = desc.workCharacterUpgrade or ""

  if self.dropMinDelay <= 0 or self.dropMaxDelay <= 0 then
   self.dropEnabled = false
  end

  if self.dropFloorBehind == true then self.dropFloor = self.floor - 1 end
  if self.dropWorkCharacter == "" then self.dropWorkCharacter = hero end
  if self.dropNotify == "" then self.dropNotify = nil end
end

function DropperObject:GatherSoundVariables(desc)
  self.soundOccasional = desc.soundOccasional or ""
  self.soundDropping = desc.soundDropping or ""
  self.soundWork = desc.soundWork or ""

  self.soundOccasionalMinDelay = desc.soundOccasionalMinDelay or 5000
  self.soundOccasionalMaxDelay = desc.soundOccasionalMaxDelay or 10000

  if self.soundOccasionalMaxDelay <= 0 then self.soundOccasional = "" end
end

function DropperObject:GatherOtherVariables(desc)
  self.ignoreBlocking = desc.ignoreBlocking or false -- Default false
end

function DropperObject:SetupObject()
  self.drawOutline = false

  -- Add some extra parameters to every location, specially if droppable:
  for i, v in ipairs(self.locations) do
    v.isDroppable = false
    v.currentDrops = {}
    v.maxDropsInHere = 0
    for j, w in ipairs(self.dropIndexes) do
      if (w.x == i and w.y > 0) then
        v.isDroppable = true
        v.maxDropsInHere = w.y
      end
    end
  end

  self.currentTime = 0

  self.previousTarget = {x = -99999, y = -99999}
  --self.currentTarget = self.storyLocation--self:ChooseNextTarget(false) -- Choose first target

  self.currentAction = nil
  self.previousAction = nil

  self.waitDelay = 0

  self.activityTime = 0
  self.dropTime = 0
  self.soundTime = math.Random(self.soundOccasionalMinDelay, self.soundOccasionalMaxDelay)

  self.dropWhenArrive = false

  self.dropCurrentWaitingTime = 0
  self.isDropping = false

  -- Check if max number of drops is greater than the total spots available in
  -- each location:
  if self.dropMaxNum > self:GetTotalDropSpotsAvailable() then
    self.dropMaxNum = self:GetTotalDropSpotsAvailable()
  end

  self.numDropsInScene = 0

  self.needsToDrop = false

  self.objectSet = true
end

function DropperObject:ChooseNextTarget(mustBeDroppable, hasDropsAvailable)
  -- Choose a new suitable target location (mustBeDroppable makes mandatory to for
  -- the new location to be droppable and hasDropsAvailable checks specifically if
  -- the selected location still has empty slots for dropping):
  local target = nil
  local retries = 100
  local success = false

  for i = 0, retries do
    target = self.locations[math.Random(#self.locations)]
    if (level:canWalkTo(target) and target ~= self.previousTarget) then
      if mustBeDroppable == true then
        if target.isDroppable == true then
          if hasDropsAvailable == true then
            if #target.currentDrops < target.maxDropsInHere then
              return target
            end
          else return target end
        end
      else return target end
    end
    retries = retries - 1
  end
  return nil
end

function DropperObject:ChooseNextAction()
  -- Choose a new action between wait and walk, based on their chances:
  randomFromWeightedTable({
    [self.Wait] = self.waitChance,
    [self.Walk] = self.walkChance,
  })(self)
end

function DropperObject:Wait(callback)
  self.flipHorizontal =
    (self.previousTarget and self.currentTarget and self.previousTarget.x > self.currentTarget.x)

  if (self.needsToDrop) then
    self:PrepareNewDrop()
  end

  -- Puts the object to wait and play (or not) some idle animations if any:
  self.currentAction = "Wait"
  self.waitDelay = math.Random(self.waitMinDuration, self.waitMaxDuration)
  self:PlayIdleAnimations(callback)
end

function DropperObject:Walk(mustBeDroppable, hasDropsAvailable, forcedTarget)
  -- Puts the object to walk, choosing a new target location:
  self.currentAction = "Walk"
  self.previousTarget = self.currentTarget

  if forcedTarget == nil then
    self.currentTarget = self:ChooseNextTarget(mustBeDroppable, hasDropsAvailable)
  elseif (self.storyLocation) then
    self.currentTarget = self.storyLocation
  end

  if self.currentTarget == nil then self:ChooseNextAction()
  elseif level:canWalkTo(self.currentTarget) == false then
    self:ChooseNextAction()
    return
  end

  if (self.currentTarget) then
    -- If previous action was also walking, we add a few spare millis in order to
    -- make the transition a little more smooth:
    if self.previousAction == "Walk" then
      self:addTask({class = "Task", duration = 250})
    end

    -- Run the walk task with to the new destination:
    self:addTask({class = "WalkToTask",
      target = self.currentTarget,
      playDefaultOnFinish = true,
      ignoreBlocking = self.ignoreBlocking,
      })
  end
end

function DropperObject:PrepareNewDrop()
  -- Stops all previous waiting/walking tasks
  taskSystem:cancelAllTasksFor(self)
  -- Plays the drop animation before actually dropping:
  self:addTask({class = "PlayAnimationTask",
                animation = self.dropAnimation,
                cycles = self.dropAnimationCycles,
                frameTime = self.dropAnimationFrameTime,
                playDefaultOnFinish = true,
                wait = true})
  --The following task is added to avoid a bug related with the "dropWaitingTime" variable. The highest the value of the "dropWaitingTime" variable the farther the object was dropped from its target position.
  --This happens because the dropper drops and leaves at the same time and some times the leave happens just before the dropp and the object is dropped in the middle of the way of the dropper.
  --By adding this task there is a moment between the drop and the leave so the dropp will always be in a target position. (PD: Dirty Fix)
  self:addTask({class = "Task",
                duration = 4000,
              })

  self.isDropping = true
end

function DropperObject:CreateNewDrop()
  -- Actually creates the drop IngredientButton objects and adds it to the level:
  local newDrop = level:add({
        class = "IngredientButton",
        name = "Drop#" .. (#self.currentTarget.currentDrops + 1) .. "From" .. self.name,
        defaultSection = self.dropProductSection,
        baseAnimation = self.dropProductName,
        x = self.x + self.dropOffset.x,
        y = self.y + self.dropOffset.y,
        anchor = self.dropAnchor,
        pivot = "bottom",
        floor = self.dropFloor,
        workDirection = self.dropWorkDirection,
        workDuration = self.dropWorkDuration,
        workAnimation = self.dropWorkAnimation,
        workFrameTime = self.dropWorkFrameTime,
        workCharacter = self.dropWorkCharacter,
        shadowResource = self.dropShadow,
        shadowBlendMode = self.dropShadowBlendMode,
        worth = self.dropStockWorth,
        productGroup = self.dropProductGroup,
        productPart = self.dropProductPart,
        deactivateOnWorkFinish = true,
        dropLocation = self.currentTarget,
        dropTimeout = self.dropTimeout,
        useWorkChararcterIfUgrade = self.dropUseWorkChararcterIfUgrade,
		boughtUpgrade = self.dropBoughtUpgrade,
		workCharacterUpgrade = self.dropWorkCharacterUpgrade
    })
    newDrop.workPos = {x = newDrop.left + newDrop.width / 2 + self.dropWorkOffset.x,
      y = newDrop.top + newDrop.height + self.dropWorkOffset.y}

    if (level:hasResource(self.dropProductName .. "_dropped", level.defaultSection.."_icons")) then
        newDrop.baseAnimation = self.dropProductName .. "_dropped"
    end

    if (newDrop.useWorkChararcterIfUgrade and player:hasUpgrade(newDrop.boughtUpgrade))then
  		newDrop.workCharacter = newDrop.workCharacterUpgrade
  	end

    newDrop.rotation = math.Random(self.dropMinRotation, self.dropMaxRotation)

    if (self.dropNotify) then
  	   newDrop.notifyObject = level:getSpriteExt(self.dropNotify)
  	end

    -- Overwrite the OnWorkFinish callback for the object to call it and cleans
    -- the already "worked" drops and plays the work sound if any:
    local old_defaultOnWorkFinish = newDrop.defaultOnWorkFinish
    newDrop.defaultOnWorkFinish = function(_, task, success)
      old_defaultOnWorkFinish(newDrop, task, success)
      newDrop.dropTimeout = 0
      self.numDropsInScene = self.numDropsInScene - 1
      --self:DeleteObsoleteDrops()
      self:playSample(self.soundWork)
      self.heroIsWorking = false
    end

    local old_defaultOnWorkStart = newDrop.defaultOnWorkStart
    newDrop.defaultOnWorkStart = function(_, task)
      old_defaultOnWorkStart(newDrop, task)
      local product = level:getObject(self.dropNotify)
      --self:doWrongAnimation(task)
    	--if (newDrop.notifyObject:isStockFull()) then
    		--self:playSample("no_stock:sound")
    		--hero:doWrongAnimation(task)
        self.heroIsWorking = true
    		return false
    	--end
    end

    local old_onClick = newDrop.onClick
    newDrop.onClick = function()
      if old_onClick then old_onClick(newDrop) end
      newDrop.dropTimeout = 9999999
    end

    -- Save the newly created drop to our local vector:
    table.insert(self.currentTarget.currentDrops, newDrop)

    self.numDropsInScene = self.numDropsInScene + 1

    -- Play a sound while dropping (if defined in the editor):
    self:playSample(self.soundDropping)

    -- Choose the new drop delay whenever it has to perform another one:
    self.dropDelay = math.Random(self.dropMinDelay, self.dropMaxDelay)
    self.dropTime = 0
    self.isDropping = false

    self.needsToDrop = false

    self:Walk()
    self:onDropDone()
end

function DropperObject:DeleteObsoleteDrops()
  -- Traverse all the drops and delete the ones that are already decayed:
  for _, v in ipairs(self.locations) do
    for i, w in ipairs(v.currentDrops) do
      w.dropTimeout = w.dropTimeout - self.currentTime
      if w.dropTimeout <= 0 then
        w.obsolete = true -- Remove the drop from the level
        table.remove(v.currentDrops, i)
      end
    end
  end
end

function DropperObject:PlayIdleAnimations(callback)
  local totalTime = self.waitDelay
  repeat
    -- Select from the random idle animations pool and build a chained task
    -- to play them during the "Wait" state of the object. The whole animation
    -- chain will last the total waiting duration at most:
    local idleDelay = math.Random(self.idleMinDelay, self.idleMaxDelay)
    if math.Random(100) < self.idleAnimChance then
      local chosenIdle = math.Random(3)

      self:addTask({class = "PlayAnimationTask",
        animation = self.idleAnimations[chosenIdle].animation,
        cycles = self.idleAnimations[chosenIdle].cycles,
        frameTime = self.idleAnimations[chosenIdle].frameTime,
        playDefaultOnFinish = false,
        wait = true})
    else
        -- If no animation chosen, then we put the object to just wait:
        self:addTask({class = "Task", duration = idleDelay})
    end
    totalTime = totalTime - idleDelay
  until (totalTime <= 0)

  if callback then
    self:addTask({class = "CallTask", func=function() callback() end})
  end
end

function DropperObject:GetTotalDropsInScene()
  -- Returns the total number of drops currently placed in the level:
  local totalDrops = 0
  for _, v in ipairs(self.locations) do
    if v.isDroppable == true then totalDrops = totalDrops + #v.currentDrops end
  end
  return totalDrops
end

function DropperObject:GetTotalDropSpotsAvailable()
  -- Returns the total number of free spots traversing all the locations:
  local totalDropSpots = 0
  for _, v in ipairs(self.locations) do
    if v.isDroppable == true then
      totalDropSpots = totalDropSpots + (v.maxDropsInHere - #v.currentDrops)
    end
  end
  return totalDropSpots
end
