-- ************************* Fish Pound Station **************************** --

FishPond = deriveLuaClass("FishPond", "Table")

function FishPond:init(desc)
    FishPond.superInit(self, desc)

    self.thinkAnimation = desc.thinkAnimation or "think"
    self.thinkFrameTime = desc.thinkFrameTime or 150
    self.eatAnimation = desc.eatAnimation or "eat"
    self.eatDuration = desc.eatDuration or 4000
    self.eatFrameTime = desc.eatFrameTime or 150
    self.trickAnimation = desc.trickAnimation or "trick"
    self.trickDuration = desc.trickDuration or 1000
    self.trickFrameTime = desc.trickFrameTime or 150

    self.appearSound = desc.appearSound
    self.startEatingSound = desc.startEatingSound

    self.totalTime = 0
    self.originalEatDuration = 0
    self.charactersHidden = false

    self.STATE_NOTHING = 0
    self.STATE_EATING = 1
    self.STATE_TRICK = 2
    self.animationState = self.STATE_NOTHING

    self.canBeClicked = true
end

function FishPond:onLevelInitialized()
	FishPond.super.onLevelInitialized(self)

	-- Save the data of the seats so later we can use it to set the customer
    -- position to one random chair
	self.seats = {}
    local firstX = nil
    local firstY = nil
	for i=1,self.numChairs do
		local s = self:getChair(i)
		table.push_back(self.seats, s)
        if (firstX == nil) then
            firstX = s.x
            firstY = s.y
        end
		s.seatOffset = {x = firstX - s.x, y = firstY - s.y}
		s.frame = 1
	end

    self.canBeClicked = true
end


function FishPond:onTick(time)
    self:tick(time)
    self.totalTime = self.totalTime + time

    if (self.animationState == self.STATE_EATING) then
        if (self.totalTime >= self.eatDuration) then
            self.animationState = self.STATE_TRICK
            self.totalTime = 0
            -- Play the trick animation
            for _,c in ipairs(self.group.customers) do
                c:playAnimation(self.trickAnimation,1,self.trickFrameTime)
            end
        end
    elseif (self.animationState == self.STATE_TRICK) then
        if (self.totalTime >= self.trickDuration) then
            self:doneEating()
        end
    end

    if (self.group) then
        if (not self.charactersHidden) then
            self.charactersHidden = true
            local customers = self.group.customers
            for _,c in ipairs(customers) do
                c.visible = false
            end
        end
    else
        self.charactersHidden = false
    end

    if (self.group) then
        local customers = self.group.customers
        for _,c in ipairs(customers) do
            c.highlightCurrentAlpha = self.highlightCurrentAlpha
        end
    end
end

function FishPond:onStartEating()
    self.canBeClicked = false
    self.totalTime = 0
    self.animationState = self.STATE_EATING
    self.originalEatDuration = self.group.stateTimer.duration
    self.group.stateTimer.duration = 999999

    -- Play the eat animation
    for _,c in ipairs(self.group.customers) do
        c:playAnimation(self.eatAnimation,-1,self.eatFrameTime)
    end

    if (self.startEatingSound) then
        self:playSample(self.startEatingSound)
    end
end

function FishPond:onTableStateChanged()
    self.super.onTableStateChanged(self)
    if (self.plateState == "occupied" and self.group) then
        self.canBeClicked = true
        self.previousGroup = self.group
        -- Sit the customer in a random chair
        local index = math.Random(1, self.numChairs)
        for _,c in ipairs(self.group.customers) do
            c.x = c.x - self.seats[index].seatOffset.x
            c.y = c.y - self.seats[index].seatOffset.y
        end

        -- When the customer finally sits on the table, make it visible and
        -- play the think animation (if skipThink is false)
        for _,c in ipairs(self.group.customers) do
            c.visible = true
        end
        if (skipThink ~= true) then
            -- We add hideThinkingBubble to the group so the customer does the
            -- thinking animation but does not show the bubble
            self.group.hideThinkingBubble = true
            for _,c in ipairs(self.group.customers) do
                c:playAnimation(self.thinkAnimation,-1,self.thinkFrameTime)
            end
        end

        if (self.appearSound) then
            self:playSample(self.appearSound)
        end
        if (FishPond.showHint ~= nil) then
            FishPond:showHint({subject=self, id="fishArrived"})
        end
    elseif (self.plateState == "empty" and self.previousGroup) then
        for _,c in ipairs(self.previousGroup.customers) do
            c.visible = false
        end
    end
end

function FishPond:doneEating()
    self.canBeClicked = true
    self.animationState = self.STATE_NOTHING
    if (self.group) then
        self.group:onDoneEating()
        self.group:handleCustomerPay(100)

        -- Setting all the characters to invisible so we don't see them leave
        -- the restaurant
        local customers = self.group.customers
        for _,c in ipairs(customers) do
            c.visible = false
        end

        self.group.state = "Leaving"
    end
end

function FishPond:getWorkCharacter()
    -- Overwrite this function so it always returns the hero. We don't want the
    -- entertainer to be able to work in this station
    if (self.plateState == "dirty") then
        return level.cleaner or hero
    end
	return hero
end

function FishPond:onClick()
    if (self.canBeClicked) then
        self.super.onClick(self)
        -- We set the fish selected to true so we see the highlight while the hero
        -- goes to the fish
        if(self.group) then
            for _,c in ipairs(self.group.customers) do
                c.selected = true
            end
        end
    end
end

function FishPond:onWorkFinish(task)
    self.super.onWorkFinish(self, task)
    if(self.group) then
      for _,c in ipairs(self.group.customers) do
          c.selected = false
      end
    end
end
