-- ************************* Fishing Boat Station ************************* --

FishingBoat = getClass("FishingBoat")

function FishingBoat:init(desc)
	FishingBoat.superInit(self, desc)
	self.statusObjects = desc.statusObjects or {}

	self.xDisplacement = desc.xDisplacement or 1500
	self.leaveTime = desc.leaveTime or 4000
	self.outsideTime = desc.outsideTime or 2000
	self.returnTime = desc.returnTime or 4000
	self.rowBoatAnimation = desc.rowBoatAnimation or "row"
	self.animationFrameTime = desc.animationFrameTime
	self.ropeIngredient = desc.ropeIngredient
	self.ropeProduct = desc.ropeProduct
	self.waveDuration = desc.waveDuration or 8000
	self.waveAmplitude = desc.waveAmplitude or 15
	self.overSprite = desc.overSprite
	self.hideRope = desc.hideRope ~= false
	self.hideSprite = desc.hideSprite

	self.EMPTY = 0
	self.ASK_FOR_ROPE = 1
	self.MOVE_RIGHT = 2
	self.WAIT_OUTSIDE = 3
	self.MOVE_LEFT = 4
	self.animationState = self.EMPTY

	self.totalTime = 0
end

function FishingBoat:onLevelInitialized()
	FishingBoat.super.onLevelInitialized(self)

	-- Save the data of the seats so later we can use it to move the characters
	-- when the boat is moving
	self.seats = {}
	for i=1,self.numChairs do
		local s = self:getChair(i)
		table.push_back(self.seats, s)
		s.boatSeatOffset = {x = self.x - s.x, y = self.y - s.y }
		s.frame = 1
	end

	self.ropeButton = level:getObject(self.ropeIngredient)
	self.overElement = level:getObject(self.overSprite)
	-- Get the offset of the over element to move it properly
	self.overElement.overwriteHighlightOnGameplay = true;
	self.overElement.roundPos = false
	self.overElement.offsetX = self.overElement.x - self.x
	self.overElement.offsetY = self.overElement.y - self.y

	self:animate({duration=self.waveDuration,wave={y=self.y,amplitude=self.waveAmplitude,loops=-1}})
	self.overElement:animate({duration=self.waveDuration,wave={y=self.overElement.y,amplitude=self.waveAmplitude,loops=-1}})
	for i=1,self.numChairs do
		local s = self:getChair(i)
		s:animate({duration=self.waveDuration,wave={y=s.y,amplitude=self.waveAmplitude,loops=-1}})
	end

	if (self.hideSprite) then
		self.hideElement = level:getSpriteExt(self.hideSprite)
	end
end

function FishingBoat:onStartEating()
	if (self.animationState == self.EMPTY) then
		self.animationState = self.ASK_FOR_ROPE;
		self.group.state = "AskForRope"
		self.group.stateTimer.duration = 999999
		-- The function secondOrder is the one that makes the customer ask
		-- for the rope product.
		self:secondOrder(self.group, self.ropeProduct)
		if (FishingBoat.showHint ~= nil) then
            FishingBoat:showHint({subject=self, id="askForRope"})
        end
	elseif (self.animationState == self.ASK_FOR_ROPE) then
		self.animationState = self.MOVE_RIGHT
		self.group.stateTimer.duration = 999999
		-- Using the state None the customer does not lose patience. If we
		-- leave the state Eating, the customer goes to pay when we give him
		-- the second product.
		self.group.state = "None"
		self.totalTime = 0

		if (self.hideRope == true and self.hideElement) then
			--self.ropeButton.visible = false
			self.hideElement.visible = false
		end
		self.clickable = false

		self:addTask({
			class = "FishingBoatTask",
			boat = self,
			originalX = self.x,
			destinyX = self.x + self.xDisplacement,
			stageTimes = {
				self.leaveTime,
				self.outsideTime,
				self.returnTime
			},
			animationName = self.rowBoatAnimation,
			animationFrameTime = self.animationFrameTime,
			ropeButton = self.ropeButton,
			hideElement = self.hideElement
		})
	end
end

function FishingBoat:onTick(time)
	self:tick(time)
	self.totalTime = self.totalTime + time

	if (self.animationState == self.MOVE_RIGHT) then
		if (self.totalTime >= self.leaveTime) then
			self.animationState = self.WAIT_OUTSIDE
			self.totalTime = 0
		end
	elseif (self.animationState == self.WAIT_OUTSIDE) then
		if (self.totalTime >= self.outsideTime) then
			self.animationState = self.MOVE_LEFT
			self.totalTime = 0
		end
	elseif (self.animationState == self.MOVE_LEFT) then
		if (self.totalTime >= self.returnTime) then
			self.animationState = self.EMPTY
			self.group:onDoneEating()
			self.totalTime = 0
		end
	else
		-- Make the seats (and characters) move according to the movement
		-- of the boat
		self:setChairXPositions()
	end
	if (self.overElement) then
		-- Set the correct position of the over element
		self.overElement.x = self.x + self.overElement.offsetX
		self.overElement.y = self.y + self.overElement.offsetY
		-- Make the highlight of the over sprite the same as the boat
		self.overElement.highlightCurrentAlpha = self.highlightCurrentAlpha
	end
end

-- Apply positional changes to chairs
function FishingBoat:setChairXPositions()
	if (self.seats) then
		for _,s in ipairs(self.seats) do
			s.x = self.x - s.boatSeatOffset.x
			s.visible = self.visible
		end
	end
end

-- Fishing Boat task that controls the movement of the boat
LuaTask = LuaTask or getClass("LuaTask")
FishingBoatTask = deriveLuaClass("FishingBoatTask", "LuaTask")

function FishingBoatTask:init(desc)
	FishingBoatTask.superInit(self, desc)

	self.boat = desc.boat
	self.originalX = desc.originalX
	self.destinyX = desc.destinyX
	self.stageTimes = desc.stageTimes
	self.animationName = desc.animationName
	self.animationFrameTime = desc.animationFrameTime
	self.ropeButton = desc.ropeButton

	self.animationState = 0

	-- Calculate the total time of the animation
	local totalDuration = 0
	for _,v in ipairs(self.stageTimes) do
		totalDuration = totalDuration + v
	end
	self.duration = totalDuration

	-- Setting roundPos to false so the customers move properly when moving
	-- the chairs
	for _,s in ipairs(self.actor.seats) do
		local c = s:getOccupant(0)
		if (c) then
			c.roundPos = false
		end
	end
end

function FishingBoatTask:start()
	self.animationState = 1
	-- Play the animation of the characters rowing
	for _,s in ipairs(self.actor.seats) do
		local c = s:getOccupant(0)
		if (c) then
			c.shadowVisible = false
			if (self.animationFrameTime) then
				if (_ == 1) then
					c:playAnimation(self.animationName, -1, self.animationFrameTime)
				else
					--play the row animation for the right customer REVERSED
					c:playAnimation(self.animationName, -1, self.animationFrameTime, "default", nil--[[flipped]], false--[[playDuringStory]], "Reversed")
				end
			else
				if (_ == 1) then
					c:playAnimation(self.animationName, -1)
				else
					--play the row animation for the right customer REVERSED
					c:playAnimation(self.animationName, -1, 150, "default", nil--[[flipped]], false--[[playDuringStory]], "Reversed")
				end
			end
		end
	end
end

function FishingBoatTask:update(frameTime)
	local finished = FishingBoatTask.super.update(self, frameTime)

	-- Calculate the current time of this stage
	local currentTime = self.timer.time
	for i=1,(self.animationState-1),1 do
		currentTime = currentTime - self.stageTimes[i]
	end
	local currentFraction = currentTime / self.stageTimes[self.animationState]

	if (self.animationState == 1) then
		self:moveBoatToDestiny(self.originalX, self.destinyX, currentFraction)
		if (currentFraction > 1) then
			self.animationState = 2
		end
	elseif (self.animationState == 2) then
		if (currentFraction > 1) then
			self.animationState = 3
			-- Play the inverse animation of the characters rowing
			for _,s in ipairs(self.actor.seats) do
				local c = s:getOccupant(0)
				if (c) then
					c.shadowVisible = false
					if (self.animationFrameTime) then
						if (_ == 1) then
							c:playAnimation(self.animationName, -1, self.animationFrameTime, "default", true--[[flipped]])
						else
							c:playAnimation(self.animationName, -1, self.animationFrameTime, "default", true--[[flipped]], false--[[playDuringStory]], "Reversed")
						end
					else
						if (_ == 1) then
							c:playAnimation(self.animationName, -1, 150, "default", true--[[flipped]])
						else
							c:playAnimation(self.animationName, -1, 150, "default", true--[[flipped]], false--[[playDuringStory]], "Reversed")
						end
					end
				end
			end
		end
	elseif (self.animationState == 3) then
		self:moveBoatToDestiny(self.destinyX, self.originalX, currentFraction)
	end
	return finished
end

function FishingBoatTask:moveBoatToDestiny(originalX, destinyX, timeFraction)
	if (timeFraction > 1) then
		timeFraction = 1
	end
	self.boat.x = math.lerp(timeFraction, originalX, destinyX) --+ math.abs(math.sin(currentFraction * numStrokes * math.pi)) * strokeAmplitude
	self.boat:setChairXPositions()
	return true
end

function FishingBoatTask:finish()
	--self.ropeButton.visible = true
	if (self.hideElement) then
		self.hideElement.visible = true
	end
	self.boat.clickable = true
	for _,s in ipairs(self.actor.seats) do
		local c = s:getOccupant(0)
		if (c) then
			c.shadowVisible = true
		end
	end
end

function FishingBoatTask:cancel()
end

function FishingBoatTask:skip()
end
