-- ******************************* Pinata ********************************** --

Pinata = deriveLuaClass("Pinata", "Object")

-- Pinata animation states
-- 0 -> the Pinata is invisible
-- 1 -> the Pinata is visible
-- 2 -> the Pinata has stopped and the customer is playing the hit animation
-- 3 -> the Pinata is playing the break Animation
-- 4 -> the customers are playing the happy animation

function Pinata:init(desc)
    Pinata.superInit(self, desc)
    self.pinataCandy = desc.pinataCandy or "pinata_candy"

    -- Customer animations
    self.customerTapAnimation = desc.customerTapAnimation or "pinyata_tap"
    self.customerHitAnimation = desc.customerHitAnimation or "pinyata_hit"
    self.happyAnimation = desc.happyAnimation or "sit"
    self.delayBetweenTaps = desc.delayBetweenTaps or 450

    -- Pinata animations
    self.pinataIdleAnimation = desc.pinataIdleAnimation or "pinata_tap" -- posar el idle
    self.pinataIdleFrameTime = desc.pinataIdleFrameTime or 50
    self.pinataTapAnimation = desc.pinataTapAnimation or "pinata_tap"
    self.pinataTapFrameTime = desc.pinataTapFrameTime or 50
    self.pinataHitAnimation = desc.pinataHitAnimation or "pinata_hit"
    self.pinataHitFrameTime = desc.pinataHitFrameTime or 50

    -- State durations
    self.customerHitDuration = desc.customerHitDuration or 500
    self.pinataHitDuration = desc.pinataHitDuration or 1200
    self.candyFallDuration = desc.candyFallDuration or 1600
    self.customerHappyDuration = desc.customerHappyDuration or 1500

    -- Sounds
    self.hitSound = desc.hitSound
    self.breakSound = desc.breakSound
    self.happySound = desc.happySound

    -- Decrease parameters
    self.progressBarLocation = desc.progressBarLocation
    self.stepsNumber = desc.stepsNumber or 4
    self.actualProgress = 0
    self.allowDecreaseBar = desc.allowDecreaseBar or true
    self.startDecreaseTime = desc.startDecreaseTime or 1000
    self.decreaseSpeed = desc.decreaseSpeed or 1

    self:createProgressBar()
    -- Set the onClick function of the back part of the progress bar
    local pinata = self
    self.progressBar.back.onClick = function()
        pinata:onClick()
    end

    self.station = nil
    self.pinataSpriteName = ""
    self.totalTime = 0
    self.tapTime = self.delayBetweenTaps
    self.idleTime = 0 -- resets to 0 when the user interacts with the Pinata
    self.playingIdle = true

    self.STATE_HIDDEN = 0
    self.STATE_VISIBLE = 1
    self.STATE_CUSTOMER_HIT = 2
    self.STATE_PINATA_HIT = 3
    self.STATE_BREAK = 4
    self.STATE_CUSTOMER_HAPPY = 5
end

function Pinata:onTick(time)
    self:tick(time)
    self.totalTime = self.totalTime + time
    self.idleTime = self.idleTime + time
    self.tapTime = self.tapTime + time

    if (self.animationState == self.STATE_VISIBLE) then
        if (self.allowDecreaseBar) then
            self:decreaseBar(time)
        end
        if (not self.playingIdle and self.tapTime >= self.delayBetweenTaps) then
            if (self.hitSound) then
                self:playSample(self.hitSound)
            end
            self.baseAnimation = self.pinataIdleAnimation .. "_" .. self.pinataSpriteName
            self:playAnimation(self.baseAnimation, -1, self.pinataIdleFrameTime)
            self.playingIdle = true
        end
    elseif (self.animationState == self.STATE_CUSTOMER_HIT) then
        if (self.totalTime >= self.customerHitDuration) then
            self:moveHitPinata()
        end
    elseif (self.animationState == self.STATE_PINATA_HIT) then
        if (self.totalTime >= self.pinataHitDuration) then
            self:breakPinata()
        end
    elseif (self.animationState == self.STATE_BREAK) then
        -- Play the falling candy animation
        if (self.totalTime >= self.candyFallDuration) then
            self:happyPinata()
        end
    elseif (self.animationState == self.STATE_CUSTOMER_HAPPY) then
        if (self.totalTime >= self.customerHappyDuration) then
            self:hidePinata()
            self.station:doneEating()
        end
    end

    -- Make the highlight of the customer the same that the station has
    if (self.station) then
        if (self.station.group) then
            local customers = self.station.group.customers
            for _,c in ipairs(customers) do
                c.highlightCurrentAlpha = self.station.highlightCurrentAlpha
            end
        end
    end
end

function Pinata:decreaseBar(time)
    if (self.idleTime > self.startDecreaseTime) then
        -- If the player hasn't clicked for startDecreaseTime then decrease the progress.
        if (self.actualProgress > 0 and Pinata.showHint ~= nil) then
            -- If there was progress and it's decreasing show the hint.
            Pinata:showHint({subject=self, id="decreaseBar"})
        end
        self.actualProgress = self.actualProgress - time * self.decreaseSpeed /10000
        if (self.actualProgress < 0) then
            self.actualProgress = 0
        end
        self.progressBar:setProgress(self.actualProgress)
    end
end

function Pinata:showPinata(station, pinataSpriteName)
    self.station = station
    self.idleTime = 0
    self.pinataSpriteName = pinataSpriteName
    self.baseAnimation = self.pinataIdleAnimation .. "_" .. self.pinataSpriteName
    self:playAnimation(self.baseAnimation, -1, self.pinataIdleFrameTime)
    self.animationState = self.STATE_VISIBLE
    self.visible = true
    self.clickable = true
    if (self.candy) then
        self.candy.visible = false
    end
    self:showProgressBar()
    self:restartProgressBar()

    -- Show the hint the first time the Pinata appears
    if (Pinata.showHint ~= nil) then
        Pinata:showHint({subject=self, id="showPinata"})
    end
end

function Pinata:tapPinata()
    if (self.tapTime >= self.delayBetweenTaps) then
        self.tapTime = 0
        self.playingIdle = false
        local customers = self.station.group.customers
        for _,c in ipairs(customers) do
            c:playAnimation(self.customerTapAnimation,1)
        end
        self.baseAnimation = self.pinataTapAnimation .. "_" .. self.pinataSpriteName
        self:playAnimation(self.baseAnimation,1, self.pinataTapFrameTime)
    end
end

function Pinata:stopPinata()
    -- Make the Pinata stop it's swinging and play the character break
    -- animation
    self.animationState = self.STATE_CUSTOMER_HIT
    self.clickable = false
    self.totalTime = 0
    self:playAnimation(self.baseAnimation,0)
    local customers = self.station.group.customers
    for _,c in ipairs(customers) do
        c:playAnimation(self.customerHitAnimation,1)
    end
    self:hideProgressBar()
end

function Pinata:moveHitPinata()
    self.animationState = self.STATE_PINATA_HIT
    self.totalTime = 0;
    self.baseAnimation = self.pinataHitAnimation .. "_" .. self.pinataSpriteName
    self:playAnimation(self.baseAnimation,1, self.pinataHitFrameTime)
    if (self.breakSound) then
        self:playSample(self.breakSound)
    end
end

function Pinata:breakPinata()
    self.animationState = self.STATE_BREAK
    if (self.candy) then
        self.candy.visible = true
        self.candy:playAnimation(self.candy.baseAnimation, 1)
    end
    self.totalTime = 0
end

function Pinata:happyPinata()
    self.animationState = self.STATE_CUSTOMER_HAPPY
    local customers = self.station.group.customers
    for _,c in ipairs(customers) do
         c:playAnimation(self.happyAnimation,-1)
    end
    self.totalTime = 0
    if (self.happySound) then
        self:playSample(self.happySound)
    end
end

function Pinata:hidePinata()
    self.animationState = self.STATE_HIDDEN
    self.visible = false
    self.clickable = false
    if (self.candy) then
        self.candy.visible = false
    end
    self:hideProgressBar()
end

function Pinata:onClick()
    -- Reset the idle time and update the progress
    self.idleTime = 0
    self.actualProgress = self.actualProgress + 1/self.stepsNumber
    self.progressBar:setProgress(self.actualProgress)

    if (self.actualProgress >= 1) then
        -- If the progress is 1 or greater, we have finished!
        self:stopPinata()
        self.station:stopClickOverwrite()
    else
        self:tapPinata()
    end
end
-- #######################
-- Copied from Event class
function Pinata:createProgressBar()
        self.progressBar = level:add(blueprints["progressbar_pinata"])

		self.progressBar.location = self.progressBarLocation or {x=0,y=0}

        self.progressBar.x = self.x + self.progressBar.location.x
        self.progressBar.y = self.y + self.progressBar.location.y

		self.progressBar.floor = 1000--self.worldFloor-self.progressBar.worldY+1
		self.progressBar:setProgress(0)

        self.progressBar.alpha = 0
        self.progressBar.back.clickable = false
end

function Pinata:restartProgressBar()
    if (self.progressBar) then
        self.progressBar:setProgress(0)
        self.actualProgress = 0
    end
end

function Pinata:showProgressBar()
    if (self.progressBar) then
        self.progressBar:animate({alpha=1, duration=600})
        self.progressBar.back.clickable = true
    end
end

function Pinata:hideProgressBar()
    if (self.progressBar) then
        self.progressBar:animate({alpha=0, duration=600})
        self.progressBar.back.clickable = false
    end
end
