-- ******************************* SpecialStationDrag ********************************** --

SpecialStationDrag = deriveLuaClass("SpecialStationDrag", "SpecialStationGesture")

function SpecialStationDrag:init(desc)
    SpecialStationDrag.superInit(self, desc)

    -- Editor parameters
    self.directionsArray = desc.directionsArray
    self.sizesArray = desc.sizesArray
    self.pressedSound = desc.pressedSound
    self.releasedSound = desc.releasedSound

    -- Sounds
    self.segmentCompleteSound = desc.segmentCompleteSound

    -- States (states 1 to 10 reserved for the generic gesture).
    self.STATE_MOUSE_UP = 11
    self.STATE_MOUSE_DOWN = 12
    self.actualState = self.STATE_STOP

    -- Directions
    --  8 1 2
    --  7 · 3
    --  6 5 4
    self.DIRECTION_UP = 1
    self.DIRECTION_UP_RIGHT = 2
    self.DIRECTION_RIGHT = 3
    self.DIRECTION_DOWN_RIGHT = 4
    self.DIRECTION_DOWN = 5
    self.DIRECTION_DOWN_LEFT = 6
    self.DIRECTION_LEFT = 7
    self.DIRECTION_UP_LEFT = 8

    -- Sizes
    self.SIZE_1 = 1
    self.SIZE_2 = 2
    self.SIZE_3 = 3

    -- Special threshold used to complete the gesture before it's full.
    self.DRAG_THRESHOLD = 50

    -- Internal parameters
    self.randomIndex = 1
    self.actualDragStep = 1
    self.graphicsCreated = false
    self.prevMousePos = {x=-1, y=-1}
    if (self.nextMechanic == "") then
        self.nextMechanic = nil
    end

    -- Set it's alpha to 0 so it's hidden during the shop/menu chooser screen
    self.alpha = 0

    -- Check if in the levelDesc there's the description of the directions for
    -- this drag gesture.
    if (levelDesc.dragMechanics and levelDesc.dragMechanics[self.name]) then
        self.directionsArray = levelDesc.dragMechanics[self.name]
    end

    self:createListOfDragSteps()
    if (not self.listOfDragSteps[1]) then
        self.listOfDragSteps[1] = self:createDragSteps(self.directionsArray, self.sizesArray)
    end
    if (self.graphicsCreated == false) then
        self.graphicsCreated = true
        for i=1,table.getn(self.listOfDragSteps) do
            self:createGestureGraphic(self.listOfDragSteps[i], i)
        end
        self:createDraggableButton()
    end

    if (not self.originalY) then
        self.originalY = self.y
        self.y = -2000
    end
end

function SpecialStationDrag:onLevelInitialized()
    SpecialStationDrag.super.onLevelInitialized(self)
    self:hideGraphic(false)
    if (self.nextMechanic) then
        self.nextMechanicObject = level:getObject(self.nextMechanic)
    end

    -- Control when the mouse is up even if it's outside the draggable element
    local origMouseUp = level.onMouseUp
    local selfObject = self
    level.onMouseUp = function()
        SpecialStationDrag.super.onMouseUp(selfObject)
        origMouseUp(level)
        level.draggingObject = false
        if (selfObject) then
            if (selfObject.actualState == selfObject.STATE_MOUSE_DOWN) then
                selfObject.actualState = selfObject.STATE_MOUSE_UP
                selfObject.prevMousePos.x = -1
                selfObject.prevMousePos.y = -1
                selfObject:resetStep()
                selfObject:playIdleAnimations()
                if (selfObject.buttonSprite) then
                    selfObject.buttonSprite:buttonReleased()
                end
            end
        end
    end
end

function SpecialStationDrag:onTick(time)
    self:tick(time)
    if (self.actualState == self.STATE_STOP and self.actualTime ~= nil) then
        -- This is used to move the element outside the screen when it's not
        -- used.
        self.actualTime = self.actualTime + time
        if (self.actualTime > self.appearAlphaDuration + 100) then
            if (self.y ~= -2000) then
                self.y = -2000
            end
        end
    elseif (self.actualState == self.STATE_APPEARING) then
        self.actualTime = self.actualTime + time
        if (self.actualTime >= self.appearAlphaDuration) then
            self:setState(self.STATE_MOUSE_UP)
            self.actualTime = 0
        end
    elseif (self.actualState == self.STATE_MOUSE_DOWN) then
        self:limitDragMovement(self.listOfDragSteps[self.randomIndex][self.actualDragStep].dir)
    elseif (self.actualState == self.STATE_COMPLETE_ANIMATION) then
        -- Wait for the complete animation to finish
        -- self.actualTime = self.actualTime + time
        -- if (self.actualTime >= self.completeDuration) then
        --     self:complete()
        -- end
        self:completeAnimationState(time)
    elseif (self.actualState == self.STATE_LAST_ANIMATION) then
        self:lastAnimationState(time)
    end

    -- Hint animation show
    if (self.actualState == self.STATE_MOUSE_UP) then
        self:hintControl(time)
    end

    self:controlCompleteGraphicAnimation(time)
    self:rotateSprite(self.buttonSprite, time)
end

function SpecialStationDrag:createListOfDragSteps()
    -- Creates the drag steps for all th eelements in the levelDesc.
    self.listOfDragSteps = {}
    if (levelDesc.dragList and levelDesc.dragList[self.name]) then
        local listOfSteps = levelDesc.dragList[self.name]
        local listOfSizes = levelDesc.sizeList[self.name]
        for i=1,table.getn(listOfSteps) do
            local steps = listOfSteps[i]
            local sizes = listOfSizes[i]
            self.listOfDragSteps[i] = self:createDragSteps(steps,sizes)
        end
    end
end

function SpecialStationDrag:createDragSteps(directionsArray, sizesArray)
    -- This function translates strings of directions to the constants of
    -- directions.
    local dragSteps = {}
    for i=1,table.getn(directionsArray) do
        dragSteps[i] = {dir="up", size=1}
        if (directionsArray[i] == "up") then
            dragSteps[i].dir = self.DIRECTION_UP
        elseif (directionsArray[i]  == "up_right") then
            dragSteps[i].dir = self.DIRECTION_UP_RIGHT
        elseif (directionsArray[i]  == "right") then
            dragSteps[i].dir = self.DIRECTION_RIGHT
        elseif (directionsArray[i]  == "down_right") then
            dragSteps[i].dir = self.DIRECTION_DOWN_RIGHT
        elseif (directionsArray[i]  == "down") then
            dragSteps[i].dir = self.DIRECTION_DOWN
        elseif (directionsArray[i]  == "down_left") then
            dragSteps[i].dir = self.DIRECTION_DOWN_LEFT
        elseif (directionsArray[i]  == "left") then
            dragSteps[i].dir = self.DIRECTION_LEFT
        elseif (directionsArray[i]  == "up_left") then
            dragSteps[i].dir = self.DIRECTION_UP_LEFT
        end
        if (sizesArray and sizesArray[i]) then
            dragSteps[i].size = sizesArray[i]
        end
    end
    return dragSteps
end

function SpecialStationDrag:createGestureGraphic(dragSteps, index)
    local e
    self.defaultSection = "special_stations"
    self.baseAnimation = "drag_node_complete"

    -- Create the path and the nodes
    self.nextSegmentX = 0
    self.nextSegmentY = 0
    self.dragPositions = self.dragPositions or {}
    self.dragPositions[index] = {}
    self.dragPositions[index][1] = { x=self.nextSegmentX, y=self.nextSegmentY }
    for i=1,table.getn(dragSteps) do
        e = self:createSegment(self.nextSegmentX,self.nextSegmentY,dragSteps[i].dir, dragSteps[i].size)
        e.edge:playAnimation(e.edge.baseAnimation,-1,150)
        if (i == 1) then
            -- If it's the first node, put the complete sprite.
            e.node.image = "drag_node_complete:special_stations"
        end
        self.dragPositions[index][i].edge = e.edge
        self.dragPositions[index][i].completeEdge = e.completeEdge
        self.dragPositions[index][i].node = e.node
        self.dragPositions[index][i+1] = { x=self.nextSegmentX, y=self.nextSegmentY }
    end
    -- Create the last node
    self.dragPositions[index][table.getn(dragSteps)+1].node = self:newChild({class="Sprite", image="drag_node:special_stations",
        anchor="center", x=self.nextSegmentX, y=self.nextSegmentY
    })
end

function SpecialStationDrag:createDraggableButton()
    -- Create the Draggable button
    self.buttonSprite = self:newChild({
        class="SpecialStationDragButton",
        baseAnimation="drag_button",
        defaultSection="special_stations",
        anchor="center",
        doHighlight=false,
        floor=self.floor,
        pressedSound=self.pressedSound,
        releasedSound=self.releasedSound
    })
    -- Add the label with DRAG text
    self:createActionLabel()
    self.buttonSprite.dragLabel = self.actionLabel
    self.buttonSprite:setDrag(self)
end

function SpecialStationDrag:limitDragMovement(direction)
    -- This function limits the movement of the arrow to inside the edges.
    mousePos = GH.getMouseCursorPos()
    local posX = self.buttonSprite.x
    local posY = self.buttonSprite.y
    -- For some reason, on iPads, on the last frame before releasing
    -- the touch, the mousePos returns x=-19998 and y=-19998. We
    -- set a limit to the values so it never returns a number too small,
    -- and also reset the previous mouse position.
    local absX = math.abs(mousePos.x)
    local absY = math.abs(mousePos.y)
    local sum = absX + absY
    if (sum > 9000) then
        self.prevMousePos.x = -1
        self.prevMousePos.y = -1
        -- Don't do anything more. This mouse values are incorrect.
        return
    end

    mousePos.x = mousePos.x / level.gameLayer.scaleX
    mousePos.y = mousePos.y / level.gameLayer.scaleY
    if (self.prevMousePos.x == -1 and self.prevMousePos.y == -1) then
        self.prevMousePos.x = mousePos.x
        self.prevMousePos.y = mousePos.y
    end

    local increaseStep = false
    if (direction == self.DIRECTION_UP) then
        posY = self.buttonSprite.y + (mousePos.y - self.prevMousePos.y)
        if (posY <= self.dragPositions[self.randomIndex][self.actualDragStep+1].y + self.DRAG_THRESHOLD) then
            posY = self.dragPositions[self.randomIndex][self.actualDragStep+1].y
            increaseStep = true
        elseif (posY >= self.dragPositions[self.randomIndex][self.actualDragStep].y + self.DRAG_THRESHOLD) then
            posY = self.dragPositions[self.randomIndex][self.actualDragStep].y
        end
    elseif (direction == self.DIRECTION_UP_RIGHT) then
        posX = self.buttonSprite.x + (mousePos.x - self.prevMousePos.x)
        posY = self.buttonSprite.y + (mousePos.y - self.prevMousePos.y)
        local diffX = posX - self.buttonSprite.x
        local diffY = self.buttonSprite.y - posY
        if (diffY > diffX) then
            posX = self.buttonSprite.x + diffY
        else
            posY = self.buttonSprite.y - diffX
        end
        if (posX >= self.dragPositions[self.randomIndex][self.actualDragStep+1].x) then
            posX = self.dragPositions[self.randomIndex][self.actualDragStep+1].x
            increaseStep = true
        elseif (posX <= self.dragPositions[self.randomIndex][self.actualDragStep].x) then
            posX = self.dragPositions[self.randomIndex][self.actualDragStep].x
        end
        if (posY <= self.dragPositions[self.randomIndex][self.actualDragStep+1].y) then
            posY = self.dragPositions[self.randomIndex][self.actualDragStep+1].y
            increaseStep = true
        elseif (posY >= self.dragPositions[self.randomIndex][self.actualDragStep].y) then
            posY = self.dragPositions[self.randomIndex][self.actualDragStep].y
        end
    elseif (direction == self.DIRECTION_RIGHT) then
        posX = self.buttonSprite.x + (mousePos.x - self.prevMousePos.x)
        if (posX >= self.dragPositions[self.randomIndex][self.actualDragStep+1].x - self.DRAG_THRESHOLD) then
            posX = self.dragPositions[self.randomIndex][self.actualDragStep+1].x
            increaseStep = true
        elseif (posX <= self.dragPositions[self.randomIndex][self.actualDragStep].x - self.DRAG_THRESHOLD) then
            posX = self.dragPositions[self.randomIndex][self.actualDragStep].x
        end
    elseif (direction == self.DIRECTION_DOWN_RIGHT) then
        posX = self.buttonSprite.x + (mousePos.x - self.prevMousePos.x)
        posY = self.buttonSprite.y + (mousePos.y - self.prevMousePos.y)
        local diffX = posX - self.buttonSprite.x
        local diffY = posY - self.buttonSprite.y
        if (diffY > diffX) then
            posX = self.buttonSprite.x + diffY
        else
            posY = self.buttonSprite.y + diffX
        end
        if (posX >= self.dragPositions[self.randomIndex][self.actualDragStep+1].x) then
            posX = self.dragPositions[self.randomIndex][self.actualDragStep+1].x
            increaseStep = true
        elseif (posX < self.dragPositions[self.randomIndex][self.actualDragStep].x) then
            posX = self.dragPositions[self.randomIndex][self.actualDragStep].x
        end
        if (posY >= self.dragPositions[self.randomIndex][self.actualDragStep+1].y) then
            posY = self.dragPositions[self.randomIndex][self.actualDragStep+1].y
            increaseStep = increaseStep and true
        elseif (posY <= self.dragPositions[self.randomIndex][self.actualDragStep].y) then
            posY = self.dragPositions[self.randomIndex][self.actualDragStep].y
        end
    elseif (direction == self.DIRECTION_DOWN) then
        posY = self.buttonSprite.y + (mousePos.y - self.prevMousePos.y)
        if (posY >= self.dragPositions[self.randomIndex][self.actualDragStep+1].y - self.DRAG_THRESHOLD) then
            posY = self.dragPositions[self.randomIndex][self.actualDragStep+1].y
            increaseStep = true
        elseif (posY <= self.dragPositions[self.randomIndex][self.actualDragStep].y - self.DRAG_THRESHOLD) then
            posY = self.dragPositions[self.randomIndex][self.actualDragStep].y
        end
    elseif (direction == self.DIRECTION_DOWN_LEFT) then
        posX = self.buttonSprite.x + (mousePos.x - self.prevMousePos.x)
        posY = self.buttonSprite.y + (mousePos.y - self.prevMousePos.y)
        local diffX = self.buttonSprite.x - posX
        local diffY = posY - self.buttonSprite.y
        if (diffY > diffX) then
            posX = self.buttonSprite.x - diffY
        else
            posY = self.buttonSprite.y + diffX
        end
        if (posX <= self.dragPositions[self.randomIndex][self.actualDragStep+1].x) then
            posX = self.dragPositions[self.randomIndex][self.actualDragStep+1].x
            increaseStep = true
        elseif (posX >= self.dragPositions[self.randomIndex][self.actualDragStep].x) then
            posX = self.dragPositions[self.randomIndex][self.actualDragStep].x
        end
        if (posY >= self.dragPositions[self.randomIndex][self.actualDragStep+1].y) then
            posY = self.dragPositions[self.randomIndex][self.actualDragStep+1].y
            increaseStep = true
        elseif (posY <= self.dragPositions[self.randomIndex][self.actualDragStep].y) then
            posY = self.dragPositions[self.randomIndex][self.actualDragStep].y
        end

    elseif (direction == self.DIRECTION_LEFT) then
        posX = self.buttonSprite.x + (mousePos.x - self.prevMousePos.x)
        if (posX <= self.dragPositions[self.randomIndex][self.actualDragStep+1].x + self.DRAG_THRESHOLD) then
            posX = self.dragPositions[self.randomIndex][self.actualDragStep+1].x
            increaseStep = true
        elseif (posX >= self.dragPositions[self.randomIndex][self.actualDragStep].x + self.DRAG_THRESHOLD) then
            posX = self.dragPositions[self.randomIndex][self.actualDragStep].x
        end
    elseif (direction == self.DIRECTION_UP_LEFT) then
        posX = self.buttonSprite.x + (mousePos.x - self.prevMousePos.x)
        posY = self.buttonSprite.y + (mousePos.y - self.prevMousePos.y)
        local diffX = self.buttonSprite.x - posX
        local diffY = self.buttonSprite.y - posY
        if (diffY > diffX) then
            posX = self.buttonSprite.x - diffY
        else
            posY = self.buttonSprite.y - diffX
        end
        if (posX <= self.dragPositions[self.randomIndex][self.actualDragStep+1].x) then
            posX = self.dragPositions[self.randomIndex][self.actualDragStep+1].x
            increaseStep = true
        elseif (posX > self.dragPositions[self.randomIndex][self.actualDragStep].x) then
            posX = self.dragPositions[self.randomIndex][self.actualDragStep].x
        end
        if (posY <= self.dragPositions[self.randomIndex][self.actualDragStep+1].y) then
            posY = self.dragPositions[self.randomIndex][self.actualDragStep+1].y
            increaseStep = increaseStep and true
        elseif (posY > self.dragPositions[self.randomIndex][self.actualDragStep].y) then
            posY = self.dragPositions[self.randomIndex][self.actualDragStep].y
        end
    end

    self:calculateClipRect(direction, posX, posY)

    self.buttonSprite.x = posX
    self.buttonSprite.y = posY
    self.prevMousePos.x = mousePos.x
    self.prevMousePos.y = mousePos.y

    if (increaseStep) then
        self.dragPositions[self.randomIndex][self.actualDragStep+1].node.image = "drag_node_complete:special_stations"
        self.actualDragStep = self.actualDragStep + 1
        if (table.getn(self.listOfDragSteps[self.randomIndex]) < self.actualDragStep) then
            self:goalAchieved()
        else
            self.buttonSprite:rotateArrow(self.listOfDragSteps[self.randomIndex][self.actualDragStep].dir)
        end
        if (self.segmentCompleteSound) then
            self:playSample(self.segmentCompleteSound)
        end
    end
end

function SpecialStationDrag:calculateClipRect(direction, posX, posY)
    -- This function calculates the correct clipRect for the colored edges.
    local e = self.dragPositions[self.randomIndex][self.actualDragStep].completeEdge
    e.visible = true
    local valueX, valueY, valueW, valueH
    if (direction == self.DIRECTION_UP) then
        valueX = e.x - e.anchor.y
        valueY = e.y - e.anchor.x-(e.y-posY)
        valueW = e.height
        valueH = e.y-posY
        valueH = math.min(valueH, e.width-1)
    elseif (direction == self.DIRECTION_UP_RIGHT) then
        local size = posX-e.x
        if (size < 1) then
            size = 1
        end
        valueX = e.x - e.anchor.y
        valueY = e.y - e.anchor.x-size
        valueW = size
        valueH = size
        -- e.clipRect = { x=e.x - e.anchor.y, y=e.y - e.anchor.x-size, width=size, height=size}
    elseif (direction == self.DIRECTION_RIGHT) then
        valueX = e.x - e.anchor.x
        valueY = e.y - e.anchor.y
        valueW = posX-e.x
        valueH = e.height
    elseif (direction == self.DIRECTION_DOWN_RIGHT) then
        local size = posX-e.x
        if (size < 1) then
            size = 1
        end
        valueX = e.x - e.anchor.y
        valueY = e.y - e.anchor.x
        valueW = size
        valueH = size
        -- e.clipRect = { x=e.x - e.anchor.y, y=e.y - e.anchor.x, width=size, height=size}
    elseif (direction == self.DIRECTION_DOWN) then
        valueX = e.x - e.anchor.y
        valueY = e.y - e.anchor.x
        valueW = e.height
        valueH = posY-e.y
        valueH = math.min(valueH, e.width-1)
    elseif (direction == self.DIRECTION_DOWN_LEFT) then
        local size = e.x-posX
        if (size < 1) then
            size = 1
        end
        valueX = e.x - size
        valueY = e.y - e.anchor.x
        valueW = size
        valueH = size
        -- e.clipRect = { x=e.x - size, y=e.y - e.anchor.x, width=size, height=size}
    elseif (direction == self.DIRECTION_LEFT) then
        valueX = math.floor(e.x-e.anchor.x-(e.x-posX))
        valueY = math.floor(e.y-e.anchor.y)
        valueW = math.floor(e.x-posX)
        -- If we don't use this min and e.width-1, the clipRect does some
        -- strange effects (it's shape transforms into a triangle?)
        valueW = math.min(valueW, e.width-1)
        valueH = math.floor(e.height)
    elseif (direction == self.DIRECTION_UP_LEFT) then
        local size = e.x-posX
        if (size < 1) then
            size = 1
        end
        valueX = e.x - size
        valueY = e.y - e.anchor.x-size
        valueW = size
        valueH = size
        -- e.clipRect = { x=e.x - size, y=e.y - e.anchor.x-size, width=size, height=size}
    end
    e.clipRect = { x=valueX, y=valueY, width=valueW, height=valueH }
end

function SpecialStationDrag:createSegment(x, y, direction, size)
    -- Creates the empty edge, the complete edge and the node and returns the
    -- complete edge, since it will be the only one modified by the dragging of
    -- the button.
    local e = {}
    if (direction == self.DIRECTION_UP) then
        e.edge = self:newChild({class="SpriteExt", baseAnimation="drag_edge_size" .. size, section="special_stations", anchor="left", x=x, y=y, rotation=270})
        e.completeEdge = self:newChild({class="Sprite", image="drag_edge_size" .. size .. "_complete:special_stations", anchor="left", x=x, y=y, rotation=270, visible=false})
        e.node = self:newChild({class="Sprite", image="drag_node:special_stations", anchor="center" , x=x, y=y})
        self.nextSegmentX = x + 0
        self.nextSegmentY = y - e.completeEdge.width
    elseif (direction == self.DIRECTION_UP_RIGHT) then
        e.edge = self:newChild({class="SpriteExt", baseAnimation="drag_edge_size" .. size, section="special_stations", anchor="left", x=x, y=y, rotation=315})
        e.completeEdge = self:newChild({class="Sprite", image="drag_edge_diag_size" .. size .. "_complete:special_stations", anchor="left", x=x, y=y, rotation=315, visible=false})
        e.node = self:newChild({class="Sprite", image="drag_node:special_stations", anchor="center" , x=x, y=y})
        self.nextSegmentX = x + e.completeEdge.width * math.sqrt(2)/2
        self.nextSegmentY = y - e.completeEdge.width * math.sqrt(2)/2
    elseif (direction == self.DIRECTION_RIGHT) then
        e.edge = self:newChild({class="SpriteExt", baseAnimation="drag_edge_size" .. size, section="special_stations", anchor="left", x=x, y=y})
        e.completeEdge = self:newChild({class="Sprite", image="drag_edge_size" .. size .. "_complete:special_stations", anchor="left", x=x, y=y, visible=false})
        e.node = self:newChild({class="Sprite", image="drag_node:special_stations", anchor="center", x=x, y=y})
        self.nextSegmentX = x + e.completeEdge.width
        self.nextSegmentY = y + 0
    elseif (direction == self.DIRECTION_DOWN_RIGHT) then
        e.edge = self:newChild({class="SpriteExt", baseAnimation="drag_edge_size" .. size, section="special_stations", anchor="left", x=x, y=y, rotation=45})
        e.completeEdge = self:newChild({class="Sprite", image="drag_edge_diag_size" .. size .. "_complete:special_stations", anchor="left", x=x, y=y, rotation=45, visible=false})
        e.node = self:newChild({class="Sprite", image="drag_node:special_stations", anchor="center", x=x, y=y})
        self.nextSegmentX = x + e.completeEdge.width * math.sqrt(2)/2
        self.nextSegmentY = y + e.completeEdge.width * math.sqrt(2)/2
    elseif (direction == self.DIRECTION_DOWN) then
        e.edge = self:newChild({class="SpriteExt", baseAnimation="drag_edge_size" .. size, section="special_stations", anchor="left", x=x, y=y, rotation=90})
        e.completeEdge = self:newChild({class="Sprite", image="drag_edge_size" .. size .. "_complete:special_stations", anchor="left", x=x, y=y, rotation=90, visible=false})
        e.node = self:newChild({class="Sprite", image="drag_node:special_stations", anchor="center" , x=x, y=y})
        self.nextSegmentX = x + 0
        self.nextSegmentY = y + e.completeEdge.width
    elseif (direction == self.DIRECTION_DOWN_LEFT) then
        e.edge = self:newChild({class="SpriteExt", baseAnimation="drag_edge_size" .. size, section="special_stations", anchor="left", x=x, y=y, rotation=135})
        e.completeEdge = self:newChild({class="Sprite", image="drag_edge_diag_size" .. size .. "_complete:special_stations", anchor="left", x=x, y=y, rotation=135, visible=false})
        e.node = self:newChild({class="Sprite", image="drag_node:special_stations", anchor="center" , x=x, y=y})
        self.nextSegmentX = x - e.completeEdge.width * math.sqrt(2)/2
        self.nextSegmentY = y + e.completeEdge.width * math.sqrt(2)/2
    elseif (direction == self.DIRECTION_LEFT) then
        e.edge = self:newChild({class="SpriteExt", baseAnimation="drag_edge_size" .. size, section="special_stations", anchor="left", x=x, y=y, rotation=180})
        e.completeEdge = self:newChild({class="Sprite", image="drag_edge_size" .. size .. "_complete:special_stations", anchor="left", x=x, y=y, rotation=180, visible=false})
        e.node = self:newChild({class="Sprite", image="drag_node:special_stations", anchor="center", x=x, y=y})
        self.nextSegmentX = x - e.completeEdge.width
        self.nextSegmentY = y + 0
    elseif (direction == self.DIRECTION_UP_LEFT) then
        e.edge = self:newChild({class="SpriteExt", baseAnimation="drag_edge_size" .. size, section="special_stations", anchor="left", x=x, y=y, rotation=225})
        e.completeEdge = self:newChild({class="Sprite", image="drag_edge_diag_size" .. size .. "_complete:special_stations", anchor="left", x=x, y=y, rotation=225, visible=false})
        e.node = self:newChild({class="Sprite", image="drag_node:special_stations", anchor="center", x=x, y=y})
        self.nextSegmentX = x - e.completeEdge.width * math.sqrt(2)/2
        self.nextSegmentY = y - e.completeEdge.width * math.sqrt(2)/2
    end
    return e
end

function SpecialStationDrag:showGraphic(animate)
    SpecialStationDrag.super.showGraphic(self,animate)
    self.y = self.originalY
    -- Show only the correct edges.
    for i=1,table.getn(self.dragPositions) do
        -- The last position doesn't contain edges.
        for j=1,table.getn(self.dragPositions[i])-1 do
            -- If it's the indexed element, make the edge and the node.
            self.dragPositions[i][j].edge.visible = i == self.randomIndex
            self.dragPositions[i][j].completeEdge.visible = false
            self.dragPositions[i][j].node.visible = i == self.randomIndex
        end
        self.dragPositions[i][table.getn(self.dragPositions[i])].node.visible = i == self.randomIndex
    end

    if (animate) then
        self:playShowGraphicAnimation()
    else
        self.scaleX = 1
        self.scaleY = 1
        self.alpha = 1
        self:setState(self.STATE_MOUSE_UP)
    end
    self.actualTime = 0
end

function SpecialStationDrag:setActiveGraphics()
    -- Sets the graphics to active. It's used when for the one_active mode of
    -- chaining gestures.
    self.buttonSprite:setActiveGraphics()
end

function SpecialStationDrag:setInactiveGraphics()
    -- Sets the graphics to inactive. It's used when for the one_active mode of
    -- chaining gestures.
    self.buttonSprite:setInactiveGraphics()
end

function SpecialStationDrag:startFromInactive(station)
    self:setActiveGraphics()
    self:setState(self.STATE_MOUSE_UP)
end

function SpecialStationDrag:reset()
    self.randomIndex = math.Random(table.getn(self.listOfDragSteps))
    self.actualDragStep = 1
    self.buttonSprite.x = 0
    self.buttonSprite.y = 0
    self.prevMousePos.x = -1
    self.prevMousePos.y = -1
    self.buttonSprite:rotateArrow(self.listOfDragSteps[self.randomIndex][self.actualDragStep].dir)
    self.buttonSprite.baseAnimation = "drag_button"
    self.buttonSprite.clickable = true

    if (self.actionLabel) then
        self.actionLabel.alpha = 1
    end

    local dsSize = table.getn(self.listOfDragSteps[self.randomIndex])
    for i=1,dsSize do
        if (self.dragPositions[self.randomIndex][i].completeEdge.visible) then
            self.dragPositions[self.randomIndex][i].completeEdge.visible = false
            self.dragPositions[self.randomIndex][i].node.image = "drag_node:special_stations"
        end
    end
    -- Set the image of the last node to the normal one and complete the first node
    self.dragPositions[self.randomIndex][dsSize+1].node.image = "drag_node:special_stations"
    self.dragPositions[self.randomIndex][1].node.image = "drag_node_complete:special_stations"
end

function SpecialStationDrag:completeGraphicAnimation()
    SpecialStationDrag.super.completeGraphicAnimation(self)
    self.buttonSprite.baseAnimation = "drag_button_complete"
    self.buttonSprite.clickable = false
    self.buttonSprite.scaleX = 1
    self.buttonSprite.scaleY = 1
    self.buttonSprite:rotateArrow(self.DIRECTION_RIGHT)
end

function SpecialStationDrag:resetStep()
    self:showHighlight(true)
    self.buttonSprite.x = self.dragPositions[self.randomIndex][self.actualDragStep].x
    self.buttonSprite.y = self.dragPositions[self.randomIndex][self.actualDragStep].y
    self:calculateClipRect(self.listOfDragSteps[self.randomIndex][self.actualDragStep].dir,
        self.dragPositions[self.randomIndex][self.actualDragStep].x,
        self.dragPositions[self.randomIndex][self.actualDragStep].y)
end
