-- ******************************* SpecialStationHold ********************************** --

SpecialStationHold = deriveLuaClass("SpecialStationHold", "SpecialStationGesture")

function SpecialStationHold:init(desc)
    SpecialStationHold.superInit(self, desc)

    -- Editor parameters
    self.holdDuration = desc.holdDuration or 5000
    self.decreaseStart = desc.decreaseStart or 1500
    self.decreaseSound = desc.decreaseSound

    -- Internal parameters
    self.actualHoldDuration = 0
    self.actualDecreaseDuration = 0
    self.minScale = 0.25
    if (self.nextMechanic == "") then
        self.nextMechanic = nil
    end

    -- Original data for resetting
    self.originalHoldDuration = self.holdDuration
    self.originalDecreaseStart = self.decreaseStart

    -- States (states 1 to 10 reserved for the generic gesture).
    self.STATE_MOUSE_UP = 11
    self.STATE_MOUSE_DOWN = 12
    self.actualState = self.STATE_STOP

    -- Set it's alpha to 0 so it's hidden during the shop/menu chooser screen
    self.alpha = 0
    self.playedDecreaseSound = false

    self:createGestureGraphic()
    self:createActionLabel()
end

function SpecialStationHold:onLevelInitialized()
    SpecialStationHold.super.onLevelInitialized(self)
    self:hideGraphic(false)

    if (self.nextMechanic) then
        self.nextMechanicObject = level:getObject(self.nextMechanic)
    end

    if (self.highlightButtonObject) then
        self.highlightButtonObject.clickable = true
        local selfObject = self
        self.highlightButtonObject.onClick = function()
            -- Overwrite the onClick of this object
        end
        self.highlightButtonObject.onMouseDown = function()
            selfObject:onMouseDown()
        end
        self.highlightButtonObject.onMouseUp = function()
            selfObject:onMouseUp()
        end

    end
end

function SpecialStationHold:onTick(time)
    self:tick(time)

    if (self.actualState == self.STATE_APPEARING) then
        self.actualHoldDuration = self.actualHoldDuration + time
        if (self.actualHoldDuration >= self.appearAlphaDuration) then
            self:setState(self.STATE_MOUSE_UP)
            self.actualHoldDuration = 0
        end
    elseif (self.actualState == self.STATE_MOUSE_DOWN) then
        self.actualHoldDuration = self.actualHoldDuration + time
        if (self.actualHoldDuration >= self.holdDuration) then
            -- Set the scale of the internal sprite to 1 so it fills the entire
            -- frame.
            self.internalSprite.scaleX = 1
            self.internalSprite.scaleY = 1
            self:goalAchieved()
        else
            self:calculateInternalScale()
        end
    elseif (self.actualState == self.STATE_MOUSE_UP) then
        self.actualDecreaseDuration = self.actualDecreaseDuration + time
        if (self.actualDecreaseDuration >= self.decreaseStart) then
            self.actualHoldDuration = self.actualHoldDuration - time
            if (self.actualHoldDuration <= 0) then
                self.actualHoldDuration = 0
                self:stopLoopSample()
            else
                -- Play the decrease sound only if it wasn't played
                if (not self.playedDecreaseSound and self.decreaseSound) then
                    self.playedDecreaseSound = true
                    self:loopSample(self.decreaseSound)
                end
                self.internalSprite.image = "hold_regress:special_stations"
            end
            self:calculateInternalScale()
        end
    elseif (self.actualState == self.STATE_COMPLETE_ANIMATION) then
        -- Wait for the complete animation to be completed
        -- self.actualTime = self.actualTime + time
        -- if (self.actualTime >= self.completeDuration) then
        --     self:complete()
        -- end
        self:completeAnimationState(time)
    elseif (self.actualState == self.STATE_LAST_ANIMATION) then
        self:lastAnimationState(time)
    end

    -- Hint animation show
    if (self.actualState == self.STATE_MOUSE_UP) then
        self:hintControl(time)
    end

    self:controlCompleteGraphicAnimation(time)
    self:rotateSprite(self.borderSprite, time)
end

function SpecialStationHold:createGestureGraphic()
    self.clickable = true
    self.internalSprite = self:newChild({class="Sprite", image= "hold_fill:special_stations", anchor="center"})
    local fontSetup = {font="main_font:fonts", color0=0x0000ff, color1=0x000099}
    self.borderSprite = self:newChild({
        class="Sprite", image="hold_border:special_stations", anchor="center"
    })
end

function SpecialStationHold:onMouseOut()
    SpecialStationHold.super.onMouseOut(self)
    if (self.actualState ~= self.STATE_APPEARING and self.actualState ~= self.STATE_STOP
        and self.actualState ~= self.STATE_LAST_ANIMATION
        and self.actualState ~= self.STATE_COMPLETE_ANIMATION) then
        if (self.actualState == self.STATE_MOUSE_DOWN) then
            -- If we were coming from the down, restart the decrease timer
            self.actualDecreaseDuration = 0
        end
        self:setState(self.STATE_MOUSE_UP)
        self:playIdleAnimations()
    end
end

function SpecialStationHold:onMouseDown()
    SpecialStationHold.super.onMouseDown(self)
    if (self.actualState ~= self.STATE_APPEARING and self.actualState ~= self.STATE_STOP
        and self.actualState ~= self.STATE_COMPLETE_ANIMATION) then
        self.internalSprite.image = "hold_fill:special_stations"
        self:hideHighlight(false)
        self:hideHintAnimation()
        self:setState(self.STATE_MOUSE_DOWN)
        self:stopLoopSample()
        self.playedDecreaseSound = false
        self:playActiveAnimations();
    end
end

function SpecialStationHold:onMouseUp()
    SpecialStationHold.super.onMouseUp(self)
    if (self.actualState ~= self.STATE_APPEARING and self.actualState ~= self.STATE_STOP
        and self.actualState ~= self.STATE_LAST_ANIMATION
        and self.actualState ~= self.STATE_COMPLETE_ANIMATION) then
        self.actualDecreaseDuration = 0
        self:setState(self.STATE_MOUSE_UP)
        self:playIdleAnimations()
    end
end

function SpecialStationHold:calculateInternalScale()
    local calculatedScale = self.actualHoldDuration/self.holdDuration
    -- -- calculatedScale = (1 - self.minScale) * (1 - calculatedScale) + self.minScale
    self.internalSprite.scaleX = calculatedScale
    self.internalSprite.scaleY = calculatedScale
end

function SpecialStationHold:showGraphic(animate)
    SpecialStationHold.super.showGraphic(self,animate)
    if (animate) then
        self:playShowGraphicAnimation()
    else
        self.scaleX = 1
        self.scaleY = 1
        self.alpha = 1
        self:setState(self.STATE_MOUSE_UP)
    end
    self.actualHoldDuration = 0
end

function SpecialStationHold:setActiveGraphics()
    -- Sets the graphics to active. It's used when for the one_active mode of
    -- chaining gestures.
end

function SpecialStationHold:setInactiveGraphics()
    -- Sets the graphics to inactive. It's used when for the one_active mode of
    -- chaining gestures.
end

function SpecialStationHold:startFromInactive(station)
    self:setActiveGraphics()
    self:setState(self.STATE_MOUSE_UP)
end

function SpecialStationHold:reset()
    self.actualHoldDuration = 0
    self.internalSprite.scaleX = 0
    self.internalSprite.scaleY = 0
    self.baseAnimation = "hold_border"
    self.internalSprite.image = "hold_fill:special_stations"
    self.clickable = true
    self.doHighlight = true
end

function SpecialStationHold:modifyPropertiesfromProduct(properties)
    if (properties.holdDuration) then
        self.holdDuration = properties.holdDuration
    end
    if (properties.decreaseStart) then
        self.decreaseStart = properties.decreaseStart
    end
end

function SpecialStationHold:resetProperties()
    self.holdDuration = self.originalHoldDuration
    self.decreaseStart = self.originalDecreaseStart
end

function SpecialStationHold:completeGraphicAnimation()
    SpecialStationHold.super.completeGraphicAnimation(self)
    -- self.internalSprite.image = "hold_range_complete:special_stations"
    self.doHighlight = false
    self.clickable = false
end
