-- ******************************* SpecialStationRange ********************************** --

SpecialStationRange = deriveLuaClass("SpecialStationRange", "SpecialStationGesture")

function SpecialStationRange:init(desc)
    SpecialStationRange.superInit(self, desc)

    -- Editor parameters
    self.barRotation = desc.barRotation or 0
    self.rangeSpeed = desc.rangeSpeed or 1
    self.rangeDifficulty = desc.rangeDifficulty or "easy"
    self.positiveAreaStart = desc.positiveAreaStart or 0
    self.rangeMissSound = desc.rangeMissSound
    self.rangeMinAngle = desc.rangeMinAngle or 0
    self.rangeMaxAngle = desc.rangeMaxAngle or 135
    self.rangeShiningSuffix = desc.rangeShiningSuffix or "shining" -- or nil

    self.customerLowMissAnimation = desc.customerLowMissAnimation
    self.customerLowMissLoop = desc.customerLowMissLoop or 1
    self.customerLowMissFrameTime = desc.customerLowMissFrameTime or 150
    self.customerLowMissMode = desc.customerLowMissMode or "Default"
    self.customerHighMissAnimation = desc.customerHighMissAnimation
    self.customerHighMissLoop = desc.customerHighMissLoop or 1
    self.customerHighMissFrameTime = desc.customerHighMissFrameTime or 150
    self.customerHighMissMode = desc.customerHighMissMode or "Default"

    self.workerLowMissAnimation = desc.workerLowMissAnimation
    self.workerLowMissLoop = desc.workerLowMissLoop or 1
    self.workerLowMissFrameTime = desc.workerLowMissFrameTime or 150
    self.workerLowMissMode = desc.workerLowMissMode or "Default"
    self.workerHighMissAnimation = desc.workerHighMissAnimation
    self.workerHighMissLoop = desc.workerHighMissLoop or 1
    self.workerHighMissFrameTime = desc.workerHighMissFrameTime or 150
    self.workerHighMissMode = desc.workerHighMissMode or "Default"

    self.missDuration = desc.missDuration or 500
    self.randomStartPosition = desc.randomStartPosition or false
    self.randomPositivePosition = desc.randomPositivePosition or false
    self.missRangeSprite = desc.missRangeSprite

    self.extraCircle = desc.extraCircle

    -- States (states 1 to 10 reserved for the generic gesture).
    self.STATE_INCREASING = 11
    self.STATE_DECREASING = 12
    self.STATE_WRONG_CLICK = 13
    self.actualState = self.STATE_STOP

    -- Original data for resetting
    self.originalRangeSpeed = self.rangeSpeed

    -- Internal parameters
    self.actualSpeed = self.rangeSpeed
    self.actualRangeTime = 0
    self.maxScale = 1
    self.middleScale = 0.67
    self.minScale = 0
    if (self.nextMechanic == "") then
        self.nextMechanic = nil
    end

    -- Calculate the easy, medium and hard positiveAreaStart caps
    self.easyCap = (self.rangeMaxAngle - levelDesc.rangeDifficultyAngles["easy"]) / self.rangeMaxAngle
    self.mediumCap = (self.rangeMaxAngle - levelDesc.rangeDifficultyAngles["medium"]) / self.rangeMaxAngle
    self.hardCap = (self.rangeMaxAngle - levelDesc.rangeDifficultyAngles["hard"]) / self.rangeMaxAngle

    -- Cap the value of Positive area start depending on the difficulty, so
    -- the green part isn't outside the range meter.
    self:capPositiveAreaStart()

    -- Set it's alpha to 0 so it's hidden during the shop/menu chooser screen
    self.alpha = 0

    -- For the range station, it's better if the anchor is at the bottom
    self.anchor = "bottom"

    -- Calculate the angles where the range is correct
    self.bottomCorrectAngle = self.positiveAreaStart * self.rangeMaxAngle
    self.topCorrectAngle = self.bottomCorrectAngle
        + levelDesc.rangeDifficultyAngles[self.rangeDifficulty]
    self:createGestureGraphic()
    self:createActionLabel()

    -- If we have a barRotation, move the graphic and the label appropiately
    -- so it matches the rotation 0.
    if (self.barRotation ~= 0) then
        local r1 = math.rad(self.barRotation)
        local r2 = math.rad(self.barRotation +270)
        local r3 = math.rad(self.barRotation +90)
        self.rotation = self.barRotation
        self.actionLabel.rotation = -self.rotation
        self.actionLabel.x = self.actionLabelPosition.x * math.cos(r1) + self.actionLabelPosition.y * math.cos(r2)
        self.actionLabel.y = self.actionLabelPosition.y * math.cos(r1) + self.actionLabelPosition.x * math.cos(r3)
    end

    -- Shining control
    self.shining = false
end

function SpecialStationRange:onLevelInitialized()
    SpecialStationRange.super.onLevelInitialized(self)
    self:hideGraphic(false)

    if (self.nextMechanic) then
        self.nextMechanicObject = level:getObject(self.nextMechanic)
    end

    if (self.highlightButtonObject) then
        self.highlightButtonObject.clickable = true
        local selfObject = self
        self.highlightButtonObject.onClick = function()
            -- Overwrite onClick so it does nothing
        end
        self.highlightButtonObject.onMouseDown = function()
            selfObject:onMouseDown()
        end
    end
    if (self.extraCircle) then
        self.extraCircleObject = level:getObject(self.extraCircle)
        if (self.extraCircleObject) then
            self.extraCircleObject.clickable = true
            local selfObject = self
            self.extraCircleObject.onClick = function()
                -- Overwrite onClick so it does nothing
            end
            self.extraCircleObject.onMouseDown = function()
                selfObject:onMouseDown()
            end
        end
    end
end

function SpecialStationRange:onTick(time)
    self:tick(time)

    if (self.actualState == self.STATE_APPEARING) then
        self.actualRangeTime = self.actualRangeTime + time
        if (self.actualRangeTime >= self.appearAlphaDuration) then
            self:setState(self.STATE_INCREASING)
            self.actualRangeTime = 0
        end
    elseif (self.actualState == self.STATE_INCREASING) then
        self.meterSprite.rotation = self.meterSprite.rotation + self.rangeSpeed * time/10
        self:showShiningSprite()
        if (self.meterSprite.rotation > self.rangeMaxAngle) then
            self.meterArrowsSprite.rotation = self.meterSprite.rotation
            self:setState(self.STATE_DECREASING)
        end
        self.meterArrowsSprite.rotation = self.meterSprite.rotation
    elseif (self.actualState == self.STATE_DECREASING) then
        self.meterSprite.rotation = self.meterSprite.rotation - self.rangeSpeed * time/10
        self:showShiningSprite()
        if (self.meterSprite.rotation < self.rangeMinAngle) then
            self.meterSprite.rotation = self.rangeMinAngle
            self.meterArrowsSprite.rotation = self.meterSprite.rotation
            self:setState(self.STATE_INCREASING)
        end
        self.meterArrowsSprite.rotation = self.meterSprite.rotation
    elseif (self.actualState == self.STATE_WRONG_CLICK) then
        self.actualTime = self.actualTime + time
        if (self.actualTime >= self.missDuration) then
            self.actualTime = 0
            self:resetFromWrong()
            self:setState(self.STATE_INCREASING)
            -- Show again the highlight button after failing.
            self:showHighlight(true)
            -- Play the idle animation of the customer and the worker
            if (self.customerIdleAnimation and self.station) then
                self.station:playCustomersAnimation(self.customerIdleAnimation,
                    self.customerIdleLoop, self.customerIdleFrameTime, self.customerIdleMode)
            end
            if (self.station) then
                local char = self.station:getWorkCharacter()
                if (char and self.workerIdleAnimation) then
                    char:playAnimation(self.workerIdleAnimation, self.workerIdleLoop,
                    self.workerIdleFrameTime, nil, nil, nil, self.workerIdleMode)
                end
            end
            if (self.gestureIdleSound) then
                self:loopSample(self.gestureIdleSound)
            end
        end
    elseif (self.actualState == self.STATE_COMPLETE_ANIMATION) then
        self:completeAnimationState(time)
    elseif (self.actualState == self.STATE_LAST_ANIMATION) then
        self:lastAnimationState(time)
    end

    -- Hint animation show
    if (self.actualState == self.STATE_INCREASING
        or self.actualState == self.STATE_DECREASING) then
        self:hintControl(time)
    end

    self:controlCompleteGraphicAnimation(time)

    -- Do the rotation only on those states, since all the SpecialStationRange
    -- share the same extraCircleObject.
    if (self.actualState == self.STATE_INCREASING or self.actualState == self.STATE_DECREASING
        or self.actualState == self.STATE_WRONG_CLICK or self.actualState == self.STATE_APPEARING
        or self.actualState == self.STATE_COMPLETE_ANIMATION or self.actualState == self.STATE_LAST_ANIMATION) then
            self:rotateSprite(self.extraCircleObject, time)
    end
end

function SpecialStationRange:createGestureGraphic()
    self.clickable = true
    self.baseAnimation = "range_red"
    self.greenSprite = self:newChild({
        class="Sprite", image="range_green_" .. self.rangeDifficulty .. ":special_stations",
        anchor="bottom", rotation = self.rangeMaxAngle * self.positiveAreaStart,
    })
    self.greenSpriteOriginalImage = "range_green_" .. self.rangeDifficulty .. ":special_stations"
    self.greenSpriteOriginalImage = self.greenSpriteOriginalImage
    if (self.rangeShiningSuffix) then
        self.greenSpriteShiningImage = "range_green_" .. self.rangeDifficulty  .. "_" .. self.rangeShiningSuffix .. ":special_stations"
    end
    self.meterSprite = self:newChild({
        class="Sprite", image="range_meter:special_stations", anchor="bottom",
    })
    self.borderSprite = self:newChild({
        class="Sprite", image="range_border:special_stations", anchor="bottom"
    })
    self.meterArrowsSprite = self:newChild({
        class="Sprite", image="range_meter_arrows:special_stations", anchor="bottom",
    })
    self:setRandomPositiveStart()
end

function SpecialStationRange:onMouseDown()
    SpecialStationRange.super.onMouseDown(self)
    if (self.actualState == self.STATE_INCREASING
        or self.actualState == self.STATE_DECREASING) then
        self:hideHintAnimation()
        self:hideHighlight(false)
        self.meterArrowsSprite.visible = false
        if (self.meterSprite.rotation >= self.bottomCorrectAngle
            and self.meterSprite.rotation <= self.topCorrectAngle) then
            self:hideExtraCircle(true)
            self:goalAchieved()
            if (self.station and self.station.onCorrectClickAction) then
                self.station:onCorrectClickAction()
            end
        else
            self:hideExtraCircle(true)
            self:wrongClick()
        end
    end
end

function SpecialStationRange:calculateRangeScale()
    local calculatedScale = self.actualRangeTime/5000
    calculatedScale = (1 - self.minScale) * calculatedScale + self.minScale
    if (calculatedScale < self.minScale) then
        calculatedScale = self.minScale
    elseif (calculatedScale > self.maxScale) then
        calculatedScale = self.maxScale
    end
    return calculatedScale
end

function SpecialStationRange:showGraphic(animate)
    SpecialStationRange.super.showGraphic(self,animate)
    if (animate) then
        self:playShowGraphicAnimation()

        self.extraCircleObject.scaleX = self.appearScale
        self.extraCircleObject.scaleY = self.appearScale
        if (self.extraCircleObject) then
            self.extraCircleObject:animate({scaleX=1, duration=self.appearAlphaDuration})
            self.extraCircleObject:animate({scaleY=1, duration=self.appearAlphaDuration})
        end
    else
        self.scaleX = 1
        self.scaleY = 1
        self.extraCircleObject.scaleX = 1
        self.extraCircleObject.scaleY = 1
        self.alpha = 1
        self:setState(self.STATE_INCREASING)
    end
    self.actualRangeTime = 0
    self:showExtraCircle(animate)
end

function SpecialStationRange:hideGraphic(animate)
    SpecialStationRange.super.hideGraphic(self, animate)
    self:hideExtraCircle(animate)
end

function SpecialStationRange:setActiveGraphics()
    -- Sets the graphics to active. It's used when for the one_active mode of
    -- chaining gestures.
end

function SpecialStationRange:setInactiveGraphics()
    -- Sets the graphics to inactive. It's used when for the one_active mode of
    -- chaining gestures.
end

function SpecialStationRange:startFromInactive(station)
    self:setActiveGraphics()
    self:setState(self.STATE_INCREASING)
end

function SpecialStationRange:reset()
    if (self.isInStory) then
        if (self.extraCircleObject) then
            -- If we're in story, make the extraCircleObject clickable because
            -- the Gesture Task sets all objects of the scene but the Gesture
            -- not clickable.
            self.extraCircleObject.clickable = true
        end
    end

    self:setRandomPositiveStart()
    self.meterArrowsSprite.visible = true
    if (self.randomStartPosition) then
        self.meterSprite.rotation = math.Random(self.rangeMinAngle, self.rangeMaxAngle)
        self.meterArrowsSprite.rotation = self.meterSprite.rotation
    else
        self.meterSprite.rotation = self.rangeMinAngle
        self.meterArrowsSprite.rotation = self.meterSprite.rotation
    end
    self.clickable = true
    if (self.originalBorderImage) then
        self.borderSprite.image = self.originalBorderImage
    end

    self.shining = false
    self.greenSprite.image = self.greenSpriteOriginalImage
    if (self.extraCircleObject) then
        self.extraCircleObject.alpha = 0.5
    end
end

function SpecialStationRange:resetFromWrong()
    self.meterArrowsSprite.rotation = self.meterSprite.rotation
    self.meterArrowsSprite.visible = true
    self.clickable = true
    if (self.originalBorderImage) then
        self.borderSprite.image = self.originalBorderImage
    end

    self.shining = false
    self.greenSprite.image = self.greenSpriteOriginalImage
    if (self.extraCircleObject) then
        self.extraCircleObject.alpha = 0.5
    end
end

function SpecialStationRange:modifyPropertiesfromProduct(properties)
    if (properties.rangeSpeed) then
        self.rangeSpeed = properties.rangeSpeed
    end
end

function SpecialStationRange:resetProperties()
    self.rangeSpeed = self.originalRangeSpeed
end

function SpecialStationRange:wrongClick()
    self:setState(self.STATE_WRONG_CLICK)
    self.actualRangeTime = 0
    self.actualTime = 0

    if (self.borderSprite and self.missRangeSprite) then
        self.originalBorderImage = self.borderSprite.image
        self.borderSprite.image = self.missRangeSprite
    end

    if (self.rangeMissSound) then
        self:playSample(self.rangeMissSound)
    end
    if (self.customerLowMissAnimation and self.station
        and self.meterSprite.rotation < self.bottomCorrectAngle) then
        self.station:playCustomersAnimation(self.customerLowMissAnimation, self.customerLowMissLoop,
            self.customerLowMissFrameTime, self.customerLowMissMode)
    end
    if (self.customerHighMissAnimation and self.station
        and self.meterSprite.rotation > self.topCorrectAngle) then
        self.station:playCustomersAnimation(self.customerHighMissAnimation, self.customerHighMissLoop,
            self.customerHighMissFrameTime, self.customerHighMissMode)
    end


    if (self.station) then
        local char = self.station:getWorkCharacter()
        if (char and self.workerLowMissAnimation and self.station
            and self.meterSprite.rotation < self.bottomCorrectAngle) then
            char:playAnimation(self.workerLowMissAnimation, self.workerLowMissLoop,
            self.workerLowMissFrameTime, nil, nil, nil, self.workerLowMissMode)
        end
        if (char and self.workerHighMissAnimation and self.station
            and self.meterSprite.rotation > self.topCorrectAngle) then
            char:playAnimation(self.workerHighMissAnimation, self.workerHighMissLoop,
            self.workerHighMissFrameTime, nil, nil, nil, self.workerHighMissMode)
        end
    end
    self:stopLoopSample()

    if (self.station and self.station.onWrongClickAction) then
        local isLow = self.meterSprite.rotation > self.topCorrectAngle
        self.station:onWrongClickAction(isLow)
    end
end

function SpecialStationRange:completeGraphicAnimation()
    SpecialStationRange.super.completeGraphicAnimation(self)
    self.clickable = false
end

function SpecialStationRange:capPositiveAreaStart()
    if (self.rangeDifficulty == "easy") then
        if (self.positiveAreaStart > self.easyCap) then
            self.positiveAreaStart = self.easyCap
        end
    elseif (self.rangeDifficulty == "medium") then
        if (self.positiveAreaStart > self.mediumCap) then
            self.positiveAreaStart = self.mediumCap
        end
    elseif (self.rangeDifficulty == "hard") then
        if (self.positiveAreaStart > self.hardCap) then
            self.positiveAreaStart = self.hardCap
        end
    end
end

function SpecialStationRange:setRandomPositiveStart()
    if (self.randomPositivePosition) then
        if (self.rangeDifficulty == "easy") then
            self.positiveAreaStart = math.Random(0, self.easyCap)
        elseif (self.rangeDifficulty == "medium") then
            self.positiveAreaStart = math.Random(0, self.mediumCap)
        elseif (self.rangeDifficulty == "hard") then
            self.positiveAreaStart = math.Random(0, self.hardCap)
        end
        self.bottomCorrectAngle = self.positiveAreaStart * self.rangeMaxAngle
        self.greenSprite.rotation = self.bottomCorrectAngle
        self.topCorrectAngle = self.bottomCorrectAngle
            + levelDesc.rangeDifficultyAngles[self.rangeDifficulty]
    end
end

function SpecialStationRange:showShiningSprite()
    if (self.rangeShiningSuffix) then
        if (self.meterSprite.rotation >= self.bottomCorrectAngle
            and self.meterSprite.rotation <= self.topCorrectAngle) then
            -- We are inside the green area, set the shining graphic.
            if (not self.shining) then
                self.shining = true
                self.greenSprite.image = self.greenSpriteShiningImage
                if (self.extraCircleObject) then
                    self.extraCircleObject.alpha = 1
                end
                self.actionLabel.alpha = 1
            end
        else
            -- We are outside the green area, reset the graphic.
            if (self.shining) then
                self.shining = false
                self.greenSprite.image = self.greenSpriteOriginalImage
                if (self.extraCircleObject) then
                    self.extraCircleObject.alpha = 0.5
                end
                self.actionLabel.alpha = 0.5
            end
        end
    end
end

function SpecialStationRange:showExtraCircle(animate)
    if (self.extraCircleObject) then
        if (animate) then
            self.extraCircleObject.visible = true
            self.extraCircleObject.alpha = 0
            self.extraCircleObject:animate({alpha=0.5, duration=self.appearAlphaDuration})
        else
            self.extraCircleObject.alpha = 0.5
        end
        self.visible = true
    end
end

function SpecialStationRange:hideExtraCircle(animate)
    if (self.extraCircleObject) then
        if (animate) then
            self.extraCircleObject:animate({alpha=0, duration=self.appearAlphaDuration})
        else
            self.extraCircleObject.alpha = 0
        end
    end
end
