-- ******************************* SpecialStationRub ********************************** --

SpecialStationRub = deriveLuaClass("SpecialStationRub", "SpecialStationGesture")

function SpecialStationRub:init(desc)
    SpecialStationRub.superInit(self, desc)

    -- Editor parameters
    self.rubDiameter = desc.rubDiameter or 200
    self.rubDistance = desc.rubDistance or 2000


    -- States (states 1 to 10 reserved for the generic gesture).
    self.STATE_MOUSE_UP = 11
    self.STATE_MOUSE_DOWN = 12
    self.STATE_MOUSE_OUT = 13
    self.actualState = self.STATE_STOP

    -- Internal parameters
    self.actualTime = 0
    self.actualRubDistance = 0
    self.graphicsCreated = false
    self.prevMousePos = {x=-1, y=-1}
    if (self.nextMechanic == "") then
        self.nextMechanic = nil
    end
    -- Set it's alpha to 0 so it's hidden during the menu chooser screen
    self.alpha = 0

    self:createGestureGraphic()
    self:createActionLabel()

    self.anchor = "center"
end

function SpecialStationRub:onLevelInitialized()
    SpecialStationRub.super.onLevelInitialized(self)

    self:hideGraphic(false)

    if (self.nextMechanic) then
        self.nextMechanicObject = level:getObject(self.nextMechanic)
    end

    -- Control when the mouse is up even if it's outside the rub element
    local origMouseUp = level.onMouseUp
    local selfObject = self
    level.onMouseUp = function()
        origMouseUp(level)
        SpecialStationRub.super.onMouseUp(selfObject)
        if (selfObject) then
            if (selfObject.actualState == selfObject.STATE_MOUSE_DOWN or
                selfObject.actualState == selfObject.STATE_MOUSE_OUT) then
                selfObject:setMouseUp()
            end
        end
    end
end

function SpecialStationRub:onTick(time)
    self:tick(time)

    if (self.actualState == self.STATE_APPEARING) then
        self.actualHoldDuration = self.actualHoldDuration + time
        if (self.actualHoldDuration >= self.appearAlphaDuration) then
            self:setState(self.STATE_MOUSE_UP)
            self.actualHoldDuration = 0
        end
    elseif (self.actualState == self.STATE_MOUSE_DOWN) then
        local distanceValue = self:calculateDistanceValue()
        self.actualRubDistance = self.actualRubDistance + distanceValue
        self:setGrowSpriteScale(self.actualRubDistance/self.rubDistance)
        if (self.actualRubDistance >= self.rubDistance) then
            self:goalAchieved()
        end
    elseif (self.actualState == self.STATE_COMPLETE_ANIMATION) then
        self:completeAnimationState(time)
    elseif (self.actualState == self.STATE_LAST_ANIMATION) then
        self:lastAnimationState(time)
    end

    -- Hint animation show
    if (self.actualState == self.STATE_MOUSE_UP) then
        self:hintControl(time)
    end

    self:controlCompleteGraphicAnimation(time)
    self:rotateSprite(self.borderSprite, time)
end

function SpecialStationRub:createGestureGraphic()
    self.clickable = true
    self.baseAnimation = "rub_border"
    self.growSprite = self:newChild({
        class="Sprite", image="rub_fill:special_stations", anchor="center",
        scaleX=0, scaleY=0
    })
    self.borderSprite = self:newChild({
        class="Sprite", image="rub_border:special_stations", anchor="center"
    })
end

function SpecialStationRub:showGraphic(animate)
    SpecialStationRub.super.showGraphic(self,animate)
    if (animate) then
        self:playShowGraphicAnimation()
    else
        self.scaleX = 1
        self.scaleY = 1
        self.alpha = 1
        self:setState(self.STATE_MOUSE_UP)
    end
    self.actualHoldDuration = 0
end

function SpecialStationRub:setGrowSpriteScale(value)
    if (value > 1) then
        value = 1
    elseif (value < 0) then
        value = 0
    end
    self.growSprite.scaleX = value
    self.growSprite.scaleY = value
end

function SpecialStationRub:reset()
    self.actualRubDistance = 0
    self.prevMousePos.x = -1
    self.prevMousePos.y = -1
    self.clickable = true
    self:setGrowSpriteScale(0)
end

function SpecialStationRub:onMouseDown()
    SpecialStationRub.super.onMouseDown(self)
    if (self.actualState ~= self.STATE_APPEARING and self.actualState ~= self.STATE_STOP
        and self.actualState ~= self.STATE_COMPLETE_ANIMATION) then
        self:hideHintAnimation()
        self:hideHighlight(false)
        self:setState(self.STATE_MOUSE_DOWN)
        self:playActiveAnimations()
    end
end

function SpecialStationRub:onMouseOut()
    SpecialStationRub.super.onMouseOut(self)
    if (self.actualState == self.STATE_MOUSE_DOWN) then
        self:setState(self.STATE_MOUSE_OUT)
        self.prevMousePos.x = -1
        self.prevMousePos.y = -1
        self:playIdleAnimations()
    end
end

function SpecialStationRub:onMouseOver()
    if (self.actualState == self.STATE_MOUSE_OUT) then
        self:setState(self.STATE_MOUSE_DOWN)
        self.prevMousePos.x = -1
        self.prevMousePos.y = -1
        self:playActiveAnimations()
    end
end

function SpecialStationRub:setMouseUp()
    if (self.actualState ~= self.STATE_APPEARING and self.actualState ~= self.STATE_STOP
        and self.actualState ~= self.STATE_COMPLETE_ANIMATION) then
        self:setState(self.STATE_MOUSE_UP)
        self.prevMousePos.x = -1
        self.prevMousePos.y = -1
        self:playIdleAnimations()
    end
end

function SpecialStationRub:completeGraphicAnimation()
    SpecialStationRub.super.completeGraphicAnimation(self)
    self.clickable = false
end

function SpecialStationRub:calculateDistanceValue()
    local mousePos = GH.getMouseCursorPos()
    if (self.prevMousePos.x == -1 and self.prevMousePos.y == -1) then
        self.prevMousePos.x = mousePos.x
        self.prevMousePos.y = mousePos.y
    end
    local xDif = math.abs(self.prevMousePos.x - mousePos.x)
    local yDif = math.abs(self.prevMousePos.y - mousePos.y)
    self.prevMousePos.x = mousePos.x
    self.prevMousePos.y = mousePos.y

    local result = xDif + yDif
    -- For some reason, on iPads, on the last frame before releasing
    -- the touch, the mousePos returns x=-19998 and y=-19998. We
    -- set a limit to the result so it never returns a number too big,
    -- and also reset the previous mouse position.
    if (result > 9000) then
        self.prevMousePos.x = -1
        self.prevMousePos.y = -1
        result = 0
    end

    return result
end
