-- ******************************* ThrowObjectStation ********************************** --

ThrowObjectStation = deriveLuaClass("ThrowObjectStation", "SpecialStationTable")

function ThrowObjectStation:init(desc)
    ThrowObjectStation.superInit(self,desc)

    -- Editor parameters
    self.arrowSection = desc.arrowSection
    self.arrowBaseAnimation = desc.arrowBaseAnimation
    self.startArrowPosition = desc.startArrowPosition
    self.highArrowPosition = desc.highArrowPosition
    self.correctArrowPosition = desc.correctArrowPosition
    self.lowArrowPosition = desc.lowArrowPosition
    self.arrowSpeed = desc.arrowSpeed or 1
    self.arrowDisappearDelay = desc.arrowDisappearDelay or 1000
    self.soundHit = desc.soundHit

    -- Default positions, just in case they aren't set.
    if (not self.startArrowPosition) then
        self.startArrowPosition = { x=0, y=0}
    end
    if (not self.correctArrowPosition) then
        self.correctArrowPosition = { x=500, y=0}
    end
    if (not self.highArrowPosition) then
        self.highArrowPosition = { x=500, y=-200}
    end
    if (not self.lowArrowPosition) then
        self.lowArrowPosition = { x=500, y=200}
    end

    self:createArrowGraphic()
end

function ThrowObjectStation:onLevelInitialized()
    ThrowObjectStation.super.onLevelInitialized(self)
end

function ThrowObjectStation:createArrowGraphic()
    -- Create two arrow graphics, the one that will move and the one that
    -- will stay after landing.
    self.arrowSprite = self:newChild({
        class="Sprite", image=self.arrowBaseAnimation .. ":" .. self.arrowSection, anchor="center",
    })
    self.stoppedArrowSprite = self:newChild({
        class="Sprite", image=self.arrowBaseAnimation .. ":" .. self.arrowSection, anchor="center",
    })
    self.arrowSprite.visible = false
    self.stoppedArrowSprite.visible = false
end

function ThrowObjectStation:onCorrectClickAction()
    -- Called when the Special Station Range is completed correctly.
    self.arrowSprite.visible = true
    self.arrowSprite.x = self.startArrowPosition.x
    self.arrowSprite.y = self.startArrowPosition.y
    local xPos = self.correctArrowPosition.x
    local yPos = self.correctArrowPosition.y
    self.arrowSprite.rotation = self:calculateRotation(self.arrowSprite.x, self.arrowSprite.y, xPos, yPos)
    local duration = self:calculateDistance(self.arrowSprite.x, self.arrowSprite.y, xPos, yPos) / self.arrowSpeed
    self.arrowSprite:animate(
        {x=xPos, y=yPos, duration=duration, ease="in"},
        {call=function() 
            self:showStoppedArrow(xPos, yPos, self.arrowSprite.rotation)
            if (self.soundHit) then
                self:playSample(self.soundHit)
            end
        end}
    )
end

function ThrowObjectStation:onWrongClickAction(isLow)
    -- Called when the Special Station Range gesture is failed.
    local xPos = self.highArrowPosition.x
    local yPos = self.highArrowPosition.y
    if (isLow == true) then
        xPos = self.lowArrowPosition.x
        yPos = self.lowArrowPosition.y
    end
    self.arrowSprite.visible = true
    self.arrowSprite.x = self.startArrowPosition.x
    self.arrowSprite.y = self.startArrowPosition.y
    self.arrowSprite.rotation = self:calculateRotation(self.arrowSprite.x, self.arrowSprite.y, xPos, yPos)
    local duration = self:calculateDistance(self.arrowSprite.x, self.arrowSprite.y, xPos, yPos) / self.arrowSpeed
    self.arrowSprite:animate(
        {x=xPos, y=yPos, duration=duration},
        {call=function() self:showStoppedArrow(xPos, yPos, self.arrowSprite.rotation) end}
    )
end

function ThrowObjectStation:showStoppedArrow(xPos, yPos, rotation)
    -- Show the stopped arrow only if the delay is > 0, if it's 0 it means we
    -- don't want to show it.
    if (self.arrowDisappearDelay > 0) then
        self.arrowSprite.visible = false
        self.stoppedArrowSprite.x = xPos
        self.stoppedArrowSprite.y = yPos
        self.stoppedArrowSprite.visible=true
        self.stoppedArrowSprite.rotation = rotation
        self.stoppedArrowSprite:animate(
            {delay=self.arrowDisappearDelay},
            {visible=false}
        )
    end
end

function ThrowObjectStation:calculateDistance(xOrig, yOrig, xPos, yPos)
    -- Calculates the distance between two points.
    local vector = {x=xPos-xOrig, y=yPos-yOrig}
    local powDistance = vector.x*vector.x + vector.y*vector.y
    return math.sqrt(powDistance)
end

function ThrowObjectStation:calculateRotation(xOrig, yOrig, xPos, yPos)
    -- Calculate the angle between the xPos, yPos vector and the horizontal
    -- axis.
    local hVector = {x=xPos-xOrig, y=yPos-yOrig}
    local dVector = {x=xPos-xOrig, y=0}
    local dotProduct = hVector.x*dVector.x + hVector.y*dVector.y
    local modH = hVector.x*hVector.x + hVector.y*hVector.y
    modH = math.sqrt(modH)
    local modD = dVector.x*dVector.x + dVector.y*dVector.y
    modD = math.sqrt(modD)
    local result = dotProduct / (modH * modD)
    local angle = math.acos(result)
    angle = math.deg(angle)
    if (yPos < yOrig) then
        angle = -angle
    end
    return angle
end
